from bioservices import *
from easydev import Logging

try:
    import pandas as pd
except:
    pass


__all__ = ["Taxon"]

class Taxon(Logging):
    """Utility to search for information related to a taxon
    
    Uses HGNC service to fetch information about a taxon.
    ::

        >>> from bioservices.apps.taxonomy import Taxon
        >>> t = Taxon()
        >>> t.search_by_taxon("9606")
        {'Scientific Name': 'Homo sapiens', 'taxon': '9606'}

    You can also pop up the Uniprot page using::

        t.uniprot_onweb("9606")


    A full list of taxons is available here::
    
        http://www.uniprot.org/taxonomy/?query=*&format=*


    .. versionadded:: 1.2.0
    """
    def __init__(self):
        super(Taxon, self).__init__("INFO")
        self.df = pd.DataFrame(index=[], columns=["Taxon", "Scientific Name"])
        self._eutils_service = EUtils()

    def search_by_taxon(self, taxon):
        """
        should be a string without comma (only one entry accepted")
        """
        assert isinstance(taxon,str)
        assert "," not in taxon
        ret = self._eutils_service.taxonomy(taxon)
        if ret == "\n":
            #nothing found
            pass
        else:
            res = {'taxon':taxon, 'Scientific Name':ret.Taxon.ScientificName}
            #self.df.append(res)
            return res

    def info(self, taxon, lineage=False):
        """Prints information about a Taxon

        :param str taxon: taxon identifier 
        :param bool lineage: prints lineage is set to True
        """
        ret = self._eutils_service.taxonomy(taxon)
        print("Display Name: %s" % ret.Taxon.OtherNames.Name.DispName)
        print("GenBank Common name: %s" % ret.Taxon.OtherNames.GenbankCommonName)
        print("Taxon Id: %s " % ret.Taxon.TaxId)
        if lineage:
            print("Lineage:")
            for i,x in enumerate(ret.Taxon.Lineage.split(";")):
                print(i*" "+x)

    def uniprot_onweb(self, taxon):
        """Open Uniprot taxonomy page for a given taxon

        :param str taxon: taxon identifier 
        """
        import webbrowser
        import urllib2
        try:
            urllib2.urlopen('http://www.uniprot.org/taxonomy/%s' % taxon)
            webbrowser.open("http://www.uniprot.org/taxonomy/%s" % taxon)
        except urllib2.HTTPError, e:
            print("Invalid taxon")
        except urllib2.URLError, e:
            print(e.args)





from bioservices import Kegg
#from nose import with_setup

#import easydev
#import os
#sharepath = easydev.get_shared_directory_path("bioservices") + os.sep + "data"
#sharepath = easydev.get_shared_directory_path("bioservices") + os.sep  



class _test_Kegg(Kegg):

    def __init__(self):
        super(test_Kegg, self).__init__(verbose=verbose)

        #self.k.load_mapping(sharepath+os.sep+"kegg-hsa-gene_mapping_oct_2012.dat.zip")

    def test_init(self):
        self.k.methods
        self.k.bget("hsa:7535")
        print len(self.k.pathways)
        assert self.k.organism=="hsa" #hsa (homo sapiens by default)

    def test_info(self):
        self.k.info()

    def test_properties(self):
        self.k.organisms

        self.k.organisms.entry_ids
        self.k.organisms.definitions

        self.k.databases.definitions

        self.k.pathways

    def test_bget(self):
        self.k.bget("hsa:7535")

    def test_get_pathways_by_genes(self):
        self.k.get_pathways_by_genes(["hsa:3568", "hsa:3577"])

    def test_binfo(self):
        self.k.binfo("gb")

    def _test_get_number_of_genes_by_organism(self):
        self.k.get_number_of_genes_by_organism()



    def _test_retrieveKGML(self):
        self.k.retrieveKGML(pathwayid="05210", organism="hsa", filename="test.xml")
        #"http://www.genome.jp/dbget-bin/show_pathway?hsa05210"

    def _test_get_genes_by_motifs(self):
        self.k.serv.get_genes_by_motifs(["pf:DnaJ", "ps:DNAJ_2"], 1, 10)


    def _test_get_reactions_by_pathway(self):
        self.k.get_reactions_by_pathway('path:eco00260')
        self.k.get_reactions_by_pathway("path:hsa00010")

    def _test_misc(self):
        self.k.bfind("gb E-cadherin human")
        #self.k.btit("hsa:1798")

    def _test_color_pathway_by_elements(self):
        element_id_list = [ 78, 79, 51, 47 ]
        fg_list  = [ '#ff0000', '#0000ff', '#ff0000', '#0000ff' ]
        bg_list  = [ '#ffff00', '#ffff00', '#ffcc00', '#ffcc00' ]
        self.k.color_pathway_by_elements('path:bsu00010', element_id_list, fg_list, bg_list)


    def _test_get_compounds_by_pathway(self):
        cpds = self.k.get_compounds_by_pathway("path:hsa04064") 
        assert cpds == ['cpd:C00076', 'cpd:C00165', 'cpd:C01245']


    def _test_lookfor_specy(self):
        self.k.lookfor_specy("ZAP70")



from collections import OrderedDict

__all__ = ["FASTA", "MultiFASTA"]

class MultiFASTA(object):
    """Class to manipulate several several FASTA items 


    Here, we load only one FASTA (from P43408 accession entry)  but 
    you can load as many as you want.::

        >>> mf = MultiFASTA()
        >>> mf.load_fasta("P43408")

    You can then get back your accession entries as follows ::

        >>> mf.ids
        ['P43408']
    
    And the sequences in the same order::

        >>> mf.sequences
        ['MKNKVVVVTGVPGVGGTTLTQKTIEKLKEEGIEYKMVNFGTVMFEVAKEEGLVEDRDQMRKLDPDTQKRIQKLAGRKIAEMAKESNVIVDTHSTVKTPKGYLAGLPIWVLEELNPDIIVIVETSSDEILMRRLGDATRNRDIELTSDIDEHQFMNRCAAMAYGVLTGATVKIIKNRDGLLDKAVEELISVLK']

    Each FASTA is stored in :attr:`fasta`, which is a dictionary where each
    values is an instance of :class:`FASTA`::

        >>> print(mf._fasta["P43408"].fasta)
        >sp|P43408|KADA_METIG Adenylate kinase OS=Methanotorris igneus GN=adkA PE=1 SV=2
        MKNKVVVVTGVPGVGGTTLTQKTIEKLKEEGIEYKMVNFGTVMFEVAKEEGLVEDRDQMR
        KLDPDTQKRIQKLAGRKIAEMAKESNVIVDTHSTVKTPKGYLAGLPIWVLEELNPDIIVI
        VETSSDEILMRRLGDATRNRDIELTSDIDEHQFMNRCAAMAYGVLTGATVKIIKNRDGLL
        DKAVEELISVLK

    .. versionadded: 1.2.0 maybe merged with :class:`FASTA` class

    """
    def __init__(self):
        # fetch the sequence using this attribute
        self._fasta_fetcher = FASTA()

        # an ordered dictionary to store the fasta contents
        self._fasta = OrderedDict()

    def _get_fasta(self):
        return self._fasta
    fasta = property(_get_fasta, doc="Returns all FASTA instances ")

    def _get_ids(self):
        return [f for f in self._fasta.keys()]
    ids = property(_get_ids, doc="returns list of keys/accession identifiers")

    def _get_sequences(self):
       return [f.sequence for f in self._fasta.values()]
    sequences = property(_get_sequences, doc="returns list of sequences")

    def load_fasta(self, id_):
        """Loads a single FASTA file into the dictionary

        """
        fasta = self._fasta_fetcher.get_fasta(id_)

        # create a new instance of FASTA and save fasta data
        f = FASTA()
        f._fasta = fasta

        # append in the ordered dictionary
        self._fasta[id_] = f

    def save_fasta(self, filename):
        """Save all FASTA into a file"""
        fh = open(filename, "w")
        for f in self._fasta.itervalues():
            fh.write(f.fasta)
        fh.close()

    def read_fasta(self, filename):
        """Loads several FASTA from a filename"""

        fh = open(filename, "r")
        data = fh.read()
        # we split according to ">sp|" so first element is empty and should
        # be ignored
        for thisfasta in data.split(">sp|")[1:]:
            f = FASTA()
            f._fasta = ">ap|" + thisfasta
            self._fasta[f.accession] = f

        return data


class FASTA(object):
    """Dedicated class to a unique FASTA sequence

    .. doctest::

        >>> from bioservices.apps.fasta import FASTA
        >>> f = FASTA()
        >>> f.load("P43403")
        >>> acc =f.accession    # the accession (P43403)
        >>> fasta = f.fasta     # raw FASTA string
        >>> seq = f.sequence    # the sequence itself

    .. seealso:: :class:`MultiFASTA` for multi FASTA manipulation.
    .. addedversion: 1.2.0 maybe merged with :class:`MulitFASTA` class

    """

    def __init__(self):
        self._fasta = None

    def _get_fasta(self):
        return self._fasta
    fasta = property(_get_fasta, doc="returns FASTA content")

    def _get_sequence(self):
        if self.fasta:
            return "".join(self.fasta.split("\n")[1:])
        else:
            raise ValueError("You need to load a fasta sequence first using get_fasta or read_fasta")
    sequence = property(_get_sequence, doc="returns the sequence only")

    def _get_header(self):
        if self.fasta:
            return self.fasta.split("\n")[0]
        else:
            raise ValueError("You need to load a fasta sequence first using get_fasta or read_fasta")
    header = property(_get_header, doc="returns header only")

    def _get_accession(self):
        return self.header.split("|")[1]
    accession = property(_get_accession, doc="returns accession only")

    def _get_entry(self):
         return self.header.split("|")[2].split(" ")[0]
    entry = property(_get_entry, doc="returns entry only")

    def _get_gene_name(self):
        index = self.header.index("GN=")
        return self.header[index:].split(" ")[0]
    gene_name = property(_get_gene_name, 
        doc="returns gene name from GN keyword found in the header")

    def __str__(self):
        str_ = self.fasta
        return str_

    def get_fasta(self, id_):
        """Fetches FASTA from uniprot and loads into attrbiute :attr:`fasta`
        
        :param str id_: a given uniprot identifier
        :returns: the FASTA contents

        
        """
        from bioservices import UniProt
        u = UniProt(verbose=False)
        res = u.searchUniProtId(id_, format="fasta")
        self._fasta = res[:]
        return res

    def load_fasta(self, id_):
        """Fetches FASTA from uniprot and loads into attribute :attr:`fasta`

        :param str id_: a given uniprot identifier
        :returns: nothing

        .. note:: same as :meth:`get_fasta` but returns nothing
        """
        # save fasta into attributes fasta
        res = self.get_fasta(id_)

    def save_fasta(self, data, filename):
        """Save FASTA file into a filename
        
        :param str data: the FASTA contents
        :param str filename: where to save it
        """
        fh = open(filename, "w")
        fh.write(data)
        fh.close()

    def read_fasta(self, filename):
        """Reads a FASTA file and loads it
        
        Type::

            f = FASTA()
            f.read_fasta(filename)
            f.fasta

        :return: nothing
       
        .. warning:: If more than one FASTA is contained in the file, an error is raised
        """
        fh = open(filename, "r")
        data = fh.read()
        fh.close()
        if data.count(">sp|")>1:
            raise ValueError("""It looks like your FASTA file contains more than
            one FASTA. You must use MultiFASTA class instead""")
            self._fasta = data[:]




# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: uniprot.py 226 2013-07-08 07:05:14Z cokelaer $
"""Interface to some part of the UniProt web service

.. topic:: What is UniProt ?

    :URL: http://www.uniprot.org
    :Citation:

    .. highlights::

        "The Universal Protein Resource (UniProt) is a comprehensive resource for protein
        sequence and annotation data. The UniProt databases are the UniProt
        Knowledgebase (UniProtKB), the UniProt Reference Clusters (UniRef), and the
        UniProt Archive (UniParc). The UniProt Metagenomic and Environmental Sequences
        (UniMES) database is a repository specifically developed for metagenomic and
        environmental data."

        -- From Uniprot web site (help/about) , Dec 2012


.. mapping between uniprot and bunch of other DBs.
.. ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/idmapping/
.. http://www.uniprot.org/docs/speclist
.. http://www.uniprot.org/docs/pkinfam

"""
from services import RESTService
from xmltools import readXML
__all__ = ["BioCarta"]


class Panther(RESTService):
    """Interface to `BioCarta <http://www.biocarta.com>`_ pages


    HTML pages relatd to pathways.

    One can retrieve the pathways names and their list of proteins. 

        >>> from bioservics import *
        >>> b = BioCarta()
        >>> pathways = b.get_pathway_names()
        >>> proteins = b.get_pathway_protein_names(pathways[0])


    .. warning:: biocarta pathways layout can be accesses from PID

    """
    _url = "http://www.pantherdb.org/"
    def __init__(self, verbose=True):
        """**Constructor**

        :param verbose: set to False to prevent informative messages
        """
        super(Panther, self).__init__(name="BioCarta", url=Panther._url, verbose=verbose)

        self._allPathwaysURL =  "http://www.pantherdb.org/pathway/pathwayList.jsp"

    def get_pathway_names(self, startswith=""):
        """returns pathways from biocarta
        
        all human and mouse. can perform a selectiom
        """
        raise NotImplementedError
        x = readXML(self._allPathwaysURL)
        pathways = [this.get("href") for this in x.findAll("a") if "pathfiles" in this.get("href")]
        pathways =  [str(x.split("/")[-1]) for x in pathways] # split the drive
        pathways = sorted(list(set(pathways)))
        pathways = [x for x in pathways if x.startswith(startswith)]
        return pathways

    def get_biopax_pathways(self, name):
        raise NotImplementedError



from bioservices import ArrayExpress
import unittest



"""class test_AE(object):
    @classmethod
    def setup_class(klass):
        klass.e = ArrayExpress(verbose=False)
"""

class test_AE(ArrayExpress):
    def __init__(self):
        super(test_AE, self).__init__()

    @unittest.skip
    def test_retrieveFiles(self):
        #res = self.retrieveFiles(keywords="cancer+breast", species="Home+Sapiens")
        res = self.queryFiles(array="A-AFFY-33")

    def test_retrieveExperiments(self):
        #res = self.retrieveExperiments(keywords="cancer+breast", species="Home+Sapiens")

        #res = self.queryExperiments(array="A-AFFY-33")
        #assert len(res.getchildren())>0
        res = self.queryExperiments(array="A-AFFY-33", species="Homo%20Sapiens",expdesign="dose+response")
        assert len(res.getchildren())>0
        res = self.queryExperiments(array="A-AFFY-33", species="Homo%20Sapiens",expdesign="dosestupid")
        assert len(res.getchildren())==0
        res = self.queryExperiments(array="A-AFFY-33", species="Homo%20Sapiens", expdesign="dose+response", 
            sortby="releasedate", sortorder="ascending")

    def test_retrieveFile(self):
        res = self.retrieveFile("E-MEXP-31", "E-MEXP-31.idf.txt")
        self.retrieveFile("E-MEXP-31", "E-MEXP-31.idf.txt", save=True) 
        import os
        os.remove("E-MEXP-31.idf.txt")

        try:
            res = self.retrieveFile("E-MEXP-31", "E-MEXP-31.idf.txtdddd")
            assert False
        except:
            pass

    def test_format(self):
        self.format = "json"
        self.format = "xml"
        try:
            self.format = "dummy"
            assert False
        except:
            assert True

    def test_retrieveExperiment(self):
        self.retrieveExperiment("E-MEXP-31")

    @unittest.skip
    def test_extra(self):
        # works. just takes 15 seconds so let us skip it.
        res = self.e.queryFiles(keywords="cancer+breast", wholewords=True)
        res = self.e.queryFiles(keywords="cancer+breast", wholewords=True, gxa="true")
        res = self.e.queryFiles(keywords="cancer+breast", wholewords=True, directsub="false")




from bioservices import UniProt

class Peptides(object):
    """
    
    ::

        >>> p = Peptides()
        >>> p.get_fasta_sequence("Q8IYB3")
        >>> p.get_phosphosite_position("Q8IYB3", "VPKPEPIPEPKEPSPE")
        189


    Sometimes, peptides are provided with a pattern indicating the phospho site.
    e.g., ::

        >>>

    """
    def __init__(self, verbose=False):

        self.u = UniProt(verbose=verbose)

    def get_fasta_sequence(self, uniprot_name):
        seq = self.u.get_fasta_sequence(uniprot_name)
        return seq

    def get_phosphosite_position(self, uniprot_name, peptide):
        fasta = self.get_fasta_sequence(uniprot_name)
        if peptide in fasta:
            return fasta.index(peptide)
        else:
            return -1

    






from bioservices.apps.peptides import Peptides



def test_peptides():
    p = Peptides()
    pos = p.get_phosphosite_position("Q8IYB3", "VPKPEPIPEPKEPSPE")
    assert pos == 246
    p.get_fasta_sequence("Q8IYB3")




# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: kegg.py 226 2013-07-08 07:05:14Z cokelaer $
"""This module provides a class :class:`~GeneProf`

.. topic:: What is GeneProf ?

    :URL: http://www.geneprof.org
    :REST: http://ww.geneprof.org/GeneProf/api
    :Citations:  Halbritter F, Kousa AI, Tomlinson SR. GeneProf data: a resource
        of curated, integrated and reusable high-throughput genomics experiments.
        Nucleic Acids Research (2013). PubMed ID:
        `24174536 <http://www.ncbi.nlm.nih.gov/pubmed/24174536>`_, Full
        Article @ NAR.


    .. highlights::

        GeneProf is a web-based, graphical software suite that allows users to
        analyse data produced using high-throughput sequencing platforms
        (RNA-seq and ChIP-seq; "Next-Generation Sequencing" or NGS): Next-gen
        analysis for next-gen data!

       -- GeneProf home page, Nov 2013


.. versionadded:: 1.2.0

Data is freely available, under the license terms of each contributing database.

"""
from __future__ import print_function

from bioservices.services import RESTService, BioServicesError

import json

__all__ = ["GeneProf"]



class GeneProf(RESTService):
    """Interface to the `GeneProf <http://www.geneprof.org/>`_ service


    Here are some GeneProf terminology used throughout the web service
    interface:

    * Experiments are what GeneProf calls each individual data analysis project.
      An experiment typically consists of a set of input data (e.g. raw
      high-throughput sequencing reads), some experimental sample annotation, an
      analysis workflow and a selection of main outputs. More information here
      in the `Experiments section <http://www.geneprof.org/GeneProf/help_conceptsexplained.jsp#concept:Experiments>`_
    * Datasets, in GeneProf, are collections of data of the
      same type generated as the output of a component of an data analysis
      workflow. There are six generic types of datasets: FILE, SEQUENCES,
      GENOMIC_REGIONS, FEATURES, REFERENCE and SPECIAL.
      More information in `Datasets section
      <http://www.geneprof.org/GeneProf/help_conceptsexplained.jsp#concept:Datasets>`_.


    .. warning:: some of the GeneProf services requires registration.
        More about the API key registration on
        `GeneProf API page <http://www.geneprof.org/GeneProf/webapi.jsp>`_

    .. warning:: there may be a limitation on the number of request per day


    .. versionadded:: 1.2.0
    """

    _valid_format = ["json", "txt", "xml", "rdata"]
    valid_species = ["arabidopsis", "at", "ce", "celegans", "chick", "chicken",
    "danio", "dm", "dmel", "dr", "drosphila", "ef", "fruitfly", "gg", "hs",
    "hsapiens", "human", "mm", "mmusculus", "mouse", "os", "pig", "rat", "rice",
    "rn", "ss", "sscrofa", "tair", "yeast", "zebrafish"]


    def __init__(self, verbose=True):
        """.. rubric:: Constructor

        :param bool verbose: prints informative messages

        Most of the time, outputs are returned in one of the following format:
        JSON, XML, TXT or RData (R data set). The default within bioservices is
        set to JSON format. You can change the default behaviour by changing the
        attribute :attr:`default_extension`.

        .. doctest::

            >>> from bioservices import GeneProf
            >>> g = GeneProf(verbose=False)
            >>> exp = g.get_list_experiments()

        Some functionalities expect the ID of a GeneProf reference dataset as
        one input parameter (e.g., pub_hs_ens59_grch37) but aliases are
        available. The list of species alias is available in the
        :attr:`valid_species` attribute.

        """
        super(GeneProf, self).__init__(name="GeneProf",
                url="http://www.geneprof.org/GeneProf/api", verbose=verbose)
        # why is it set to False ?
        self.easyXMLConversion = False
        self._default_extension = "json"

        # buffering the ids for the experiments and datasets
        self._ids_exp = None
        self._rigid_ids_exp = None
        self._ids_ds = None
        self._rigid_ids_ds = None

    # just a get/set to the default extension
    def _set_default_ext(self, ext):
        self.checkParam(ext, ["json","xml", "txt", "rdata"])
        self._default_extension = ext
    def _get_default_ext(self):
        return self._default_extension
    default_extension = property(_get_default_ext, _set_default_ext,
             doc="""set extension of the requests (default is json). Can be
                'json' or 'xml', 'txt', 'rdata'.""")

    # buffering the ids from the experiments
    def _get_ids_exp(self):
        if self._ids_exp == None:
            self.logging.info("Fetchin ids...")
            res = self.get_list_experiments(format="json")
            self._ids_exp = [x['id'] for x in res]
            self._rigid_ids_exp = [x['rigid_id'] for x in res]
        return self._ids_exp
    ids_exp = property(_get_ids_exp)

    def _get_rigid_ids_exp(self):
        if self._rigid_ids_exp == None:
            self.logging.info("Fetchin ids...")
            res = self.get_list_experiments(format="json")
            self._rigid_ids_exp = [x['rigid_id'] for x in res]
            self._ids_exp = [x['id'] for x in res]
        return self._rigid_ids_exp
    rigid_ids_exp = property(_get_rigid_ids_exp)

    # buffering the ids from the datasets
    def _get_ids_ds(self):
        if self._ids_ds == None:
            self.logging.info("Fetchin ids...")
            res = self.get_list_reference_datasets(format="json")
            self._ids_ds = [x['id'] for x in res]
        return self._ids_ds
    ids_ds = property(_get_ids_ds)


    def get_list_experiments(self, format="json", **kargs):
        """Retrieves a list of GeneProf experiments.

        :param str format: format of the output
        :param bool with-ats: Include descriptions for all datasets'
            annotation types (data columns).
        :param bool with-samples: Include information about the sample
            annotation per experiment.
        :param bool with-inputs: Include a listing of all input datasets per
            experiment.
        :param bool with-outputs: Include a listing of the main output datasets
            per  experiment.
        :param bool with-workflow: Include the analysis workflow per experiment.
        :param bool with-all-data: Include ALL datasets linked with the
            experiment (large response!)
        :param bool only-user-experiments:  List only experiments owned by the
            user identified by the WebAPI key.
        :param bool key:  An optional WebAPI key, required to access
            non-public data.

        :return: if json output is requested, the output is a list of
            dictionaries. Each dictionary corresponds to one experiment

        .. doctest::

            from bioservices import GeneProf
            g = GeneProf(verbose=False)
            experiments = g.get_list_experiments(with_outputs=True)

        """
        
        format_ = self._check_format(format)
        url = self.url + "/exp/list." + format_

        for key in kargs.keys():
            if key not in ["with_ats", "with_samples", "with_inputs",
                    "with_outputs", "with_workflow", "with_all_data",
                    "only_user_experiments", "key"]:
                raise BioServicesError("invalid parameter (%s) provided" % key)
        params = self.urlencode(kargs)
        if len(params):
            params = params.replace("with_", "with-")
            url += "?"+ params
        res = self.request(url, format=format_)
        if format == "json":
            res = json.loads(res)
            res = res['experiments']
        return res

    def _check_format(self, format_):
        if format_ == None:
            format_ = self.default_extension
        self.checkParam(format_, self._valid_format)
        return format_

    def _check_id(self, Id):
        try:
            Id = int(Id)
        except:
            pass
        if Id not in self.ids_exp and Id not in self.rigid_ids_exp:
            raise BioServicesError("Incorrect ids provides. See ids and rigid_ids attributes.")

    def _check_reference(self, ref):
        if ref not in self.valid_species:
            raise BioServicesError("Incorrect reference. check attribute :attr:`valid_species`.")

    def _check_kargs(self, kargs, valid_keys):
        for key in kargs.keys():
            if key not in valid_keys:
                raise BioServicesError("invalid parameter (%s) provided" % key)

    def get_metadata_experiment(self, Id, format="json", **kargs):
        """Retrieves metadata (names, descriptions, IDs, etc) about experiments.

        An experiment typically consists of a set of input data (e.g. raw
        high-throughput sequencing reads), some experimental sample
        annotation, an analysis workflow and a selection of main outputs.

        :param str Id: accession ID (a string of the form gpXP_XXXXXX or a
            numeric part (e.g., 3)).

        Other parameters like in :meth:`get_list_experiments` are also available
        :return: dictionary with metadata

        Retrieve basic metadata about experiment gpXP_000385 including
        workflow::

            >>> g.get_metadata_experiment(Id="385", with_workflow=True)

        """
        format_ = self._check_format(format)
        self._check_id(Id)
        self._check_kargs(kargs, ["with_ats", "with_samples", "with_inputs",
                    "with_outputs", "with_workflow", "with_all_data", "key"])

        url = self.url + "/exp/%s." % Id + format_
        params = self.urlencode(kargs)
        if len(params):
            params = params.replace("with_", "with-")
            params = params.replace("only_user_ex", "only-user-ex")
            url += "?"+ params
        res = self.request(url, format=format_)
        if format == "json":
            res = json.loads(res)
        return res

    def get_metadata_dataset(self, Id, format="json", **kargs):
        """Get metadata about geneprof dataset

        This method retrieves metadata about a specific GeneProf dataset
        given the dataset's accession ID (a string of the form
        gpDS_XXX_XXX_XXX_XXX).

        :param str Id: The identifier of the dataset of interest. Either the
            entire accession ID (e.g. gpDS_11_385_44_1) or just the
            dataset-specific part (e.g. 11_385_44_1).
        :param str format: one of json, xml, txt, rdata

        Retrieve metadata about the dataset gpDS_11_12_122_1 as JSON::

            g.get_metadata_dataset("11_12_122_1", with_ats=True)

        """
        format_ = self._check_format(format)
        #self._check_id_ds(Id)
        self._check_kargs(kargs, ["with_ats", "key"])

        url = self.url + "/ds/%s." % Id + format_
        params = self.urlencode(kargs)
        if len(params):
            params = params.replace("with_", "with-")
            url += "?"+ params
        res = self.request(url, format=format_)
        if format == "json":
            res = json.loads(res)
        return res



    def get_list_reference_datasets(self, format="json"):
        """Retrieves a list of public reference datasets.

        GeneProf provides a number of recommended reference datasets
        for several organisms (human, mouse, etc.). These reference datasets
        provide genomic sequence assemblies and genic annotations that serve
        as a scaffold for GeneProf's analyses, so most of GeneProf's datasets
        are based on one of these reference datasets. This web service simply
        retrieves a list of all the public, recommended reference datasets
        currently available in the database.

        :param str format: format in one of: json, xml, txt, rdata.

        .. note:: the txt and rdata format versions of the output reports a
            flattened version of the dataset metadata and misses out some
            information available in the other formats!

        Retrieve a list of all reference datasets as JSON::

            >>> g.get_list_reference_datasets()

        """
        format_ = self._check_format(format)
        url = self.url + "/ds/pubref." + format_
        res = self.request(url, format=format_)
        if format_ == "json":
            res = json.loads(res)
            res = res['references']
        return res


    def get_list_experiment_samples(self, ref, format="json"):
        """Returns list of Public Experiment Samples for a reference dataset.

        All public data in the GeneProf databases has been annotated
        with the biological sample of origin, described in terms of
        cell type, tissue, treatment, and so on. This web service simply
        retrieves a list of all the public sample annotations in the database
        for a specific reference dataset (see the List Public Reference
        Datasets service).


        :param str ref: identifier of a public GeneProf reference dataset.
            you may use aliases here. Check the list public references
            service for all available reference datasets.
        :param str format: The file format requests, one of: json, xml, txt, rdata.

        Retrieve a list of all samples for mouse as XML::

            >>> g.get_list_experiment_samples("mouse")
            >>> g.get_list_experiment_samples("human", format="txt")
            >>> g.get_list_experiment_samples("human", format="rdata")

        """
        format_ = self._check_format(format)
        url = self.url + "/gene.info/list.samples/%s." % ref + format_
        res = self.request(url, format=format_)

        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
            res = res['samples']
        return res


    def search_genes(self, query, taxons=None, format="json"):
        """Search genes using genes' description, name and accession IDs.

         use sets of gene annotations
         based on those from Ensembl. You can search for
         genes of interest using arbitrarily complex search queries against the
         names and identifiers (from Ensembl, RefSeq and more) of those genes.
         The search results are categorised by the reference dataset the genes
         belong to (also see the List Public Reference Datasets service).

        :param str query: The search term to look for, e.g. a gene name or paper
             title. You can narrow down the fields to be search by prefixing the
             query with a field name. Valid fields for genes are: Valid search
             fields are: id, label, description, type and reference. You can
             also use boolean logic in your queries using the keywords AND and
             OR, brackets and quotes (") for exact matches of whole phrases.
        :param str format: file format requests, one of: json, xml, txt, rdata.
        :param str taxons: Only return matches from experiments
                     dealing with organisms matching these NCBI taxonomy IDs
                     (comma-separated list).


        (1) Search for all genes matching the query 'sox2' (in json
            format), (2) same  but only in human (taxon 9606) (3) same bu only
            in human and 10090 (4)  query "brca2 AND cancer AND reference" in
            mouse

        >>> g.search_genes("sox2")['total_results']
        8
        >>> g.search_genes("sox2", taxons="9606")['total_results']
        2
        >>> g.search_genes("sox2", taxons="9606, 10090")['total_results']
        3
        >>> res = g.search_genes("brca2 AND cancer AND reference", taxons="mouse")


        XML output example::

        >>> g.search_genes("sox2", taxons="9606", format="xml")
        >>> geneIds = [x.find('numeric_id').text for x in res.findAll("genes")]

        >>> g.search_genes("sox2", taxons="9606")
        >>> geneIds = [x["numeric_id"] for x in res[0]['genes']]


        .. seealso:: search_gene_ids
        """
        format_ = self._check_format(format)
        query = query.replace(" ", "+")
        url = self.url + "/search/gene/%s." % query + format_
        params = {}
        if taxons:
            params['taxons'] = taxons
        params = self.urlencode(params)
        if len(params):
            url += "?" +  params
        res = self.request(url, format=format_)
        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
            res = res['matches_per_dataset']
        return res

    def search_gene_ids(self, query, taxons):
        """This is an alias to :meth:`search_genes` to retrieve Ids

        geneIds = g.seqrch_gene_ids("nanog", "mouse")
        """
        results = self.search_genes(query, taxons)
        geneIds = {}
        for res in results:
            ids = [x["numeric_id"] for x in res['genes']]
            taxon = res["reference"]["taxon"]
            geneIds[taxon] = ids
        return geneIds

    def search_experiments(self, query, taxons=None, format="json"):
        """Search Experiments using  name, description and citations.

        Experiments are what GeneProf calls each individual data  analysis
        project. An experiment typically consists of a set of input data (e.g.
        raw high-throughput sequencing reads), some experimental sample
        annotation, an analysis workflow and a selection of main outputs. Please
        check the manual for further information about experiments. Using this
        web service, you can search for experiments of interest using
        arbitrarily complex search queries against the names, descriptions,
        linked citations, linked reference dataset, and so on of those
        experiments. The search results are categorised by the reference dataset
        the experiments belong to (also see the List Public Reference Datasets
        service).

        :param str query: The search term to look for, e.g. a gene name or paper
            title. You can narrow down the fields to be search by prefixing the
            query with a field name. Valid fields for experiments are: Valid
            search fields are: id, label, description, type, reference, user,
            dataset, citation, platform and sample. You can also use boolean
            logic in your queries using the keywords AND and OR, brackets and
            quotes (") for exact matches of whole phrases. Advanced search
            options and examples are documents on GeneProf's search
            page.
        :param str format: The file format requests, one of: json, xml, txt, rdata.
        :param str taxons: Only return matches from experiments dealing with
            organisms matching these NCBI taxonomy IDs (comma-separated list).

        Search for experiments mentioning 'sox2' anywhere (in XML format)::

            >>> g.search_experiments("sox2")

        Search for experiments mentioning 'cancer' in their description (in JSON
            format):

            >>> g.search_experiments("citation:cancer")

        Search for experiments mentioning 'cell stem cell' in a linked citation (in
        plain text format):

            >>> g.search_experiments("citation:'stem cell'")

        """
        format_ = self._check_format(format)
        query = query.replace(" ", "+")
        url = self.url + "/search/experiment/%s." % query + format_
        params = {}
        if taxons:
            params['taxons'] = taxons
        params = self.urlencode(params)
        if len(params):
            url += "?" +  params
        res = self.request(url, format=format_)
        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res

    def search_datasets(self, query, taxons=None, format=None):
        """search for datasets

        Using this web service, you can search for experiments of interest
        using arbitrarily complex search queries against the names and types
        of these datasets.

        :param str query: The search term to look for, e.g. a gene name
            or cell type. You can narrow down the fields to be search by
            prefixing the query with a field name. Valid fields for samples
            are: id, label, description, datatype, user, experiment .You can
            also use boolean logic in your queries using the keywords AND
            and OR, brackets and quotes (") for exact matches of whole phrases.
            Advanced search options and examples are documents on GeneProf's
            search page.
        :param str format: file format in: json, xml, txt, rdata.
        :param str taxons: Only return matches from experiments dealing
             with organisms matching these NCBI taxonomy IDs (comma-separated
             list).

         Search for datasets mentioning 'sox2'::

             >>> g.search_datasets('sox2')

         Search for datasets mentioning 'gene expression':

            >>> g.search_datasets('gene expression')

        Search for genomic data for 'sox2' in plain text format:

            >>> g.search_datasets("datatype:GENOMIC_REGIONS AND sox2")

        """
        format_ = self._check_format(format)
        query = query.replace(" ", "+")
        url = self.url + "/search/dataset/%s." % query + format_
        params = {}
        if taxons:
            params['taxons'] = taxons
        params = self.urlencode(params)
        if len(params):
            url += "?" +  params
        res = self.request(url, format=format_)
        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res


    def search_samples(self, query, taxons=None, format=None):
        """search for public experiment samples using search terms against their annotations.


        All public data in the GeneProf databases has been
        annotated with the biological sample of origin, described in terms of
        cell type, tissue, treatment, and so on. Using this web service, you can
        search for samples of interest using arbitrarily complex search queries
        against the annotations of these samples.

        :param str query: search term to look for, e.g. a gene name or cell type.
            You can narrow down the fields to be search by prefixing the
            query with a field name. Valid fields for samples are: Valid
            search fields are: id, label, description , Age, Antibody,
            Cell_Line, Cell_Type, Description, Developmental_Stage, Gender,
            Gene, Label, Organism, Platform, Sample_Group, SRA_Accession,
            Strain, Time, Tissue, Treatment. You can also use boolean logic
            in your queries using the keywords AND and OR, brackets and
            quotes (") for exact matches of whole phrases. Advanced
            search options and examples are documents on GeneProf's search
            page.
        :param bool format: file format requests, one of: json, xml, txt, rdata.
        :param str taxons: Only return matches from experiments dealing with
            organisms matching these NCBI taxonomy IDs (comma-separated list).

        Search for samples annotated 'ChIP' in any of the default search fields
        (in XML format):

            >>> g.search_samples("ChIP")

        Search for samples annotated with the gene 'sox2':

            >>> g.search_samples("Gene:sox2")

        Search for samples annotated 'human' in any of the default search fields
        in plain text format:

            >>> g.search_samples("human")

        """
        format_ = self._check_format(format)
        query = query.replace(" ", "+")
        url = self.url + "/search/sample/%s." % query + format_
        params = {}
        if taxons:
            params['taxons'] = taxons
        params = self.urlencode(params)
        if len(params):
            url += "?" +  params
        res = self.request(url, format=format_)
        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res


    def get_gene_id(self, ref, idtype, Id, format="json"):
        """Get the GeneProf ID of a Gene

        GeneProf uses well-defined sets of gene annotations
        based on those from `Ensembl <http://www.ensembl.org/>`_. you can get
        the :meth:`get_gene_id` of any gene in the reference annotation by
        matching it against an external name (official gene symbol) or one of
        the supported accession ID types (e.g. Ensembl Gene IDs, RefSeq IDs,
        etc. -- use the :meth:`get_list_idtypes` to find out which types
        are supported for a dataset).

        :param str ref: The identifier of a public GeneProf reference
            dataset. You may use aliases here. Check the list public
            references service for all available reference datasets.
        :param str idtype: The identifier of an annotation column storing IDs or
            the term any to use any available identifier type. Use
            :meth:`get_list_idtypes` to find out which types are supported for a
            dataset.
        :param str Id: The GeneProf ID of a gene (an integer number).
        :param str format: output format in json, txt, xml, rdata.

        Get the GeneProf ID of the mouse gene with Ensembl ID ENSMUSG00000059552

            >>> g.get_gene_id("mouse", "C_ENSG", "ENSMUSG00000059552")


        Get the GeneProf IDs of all human genes with RefSeq ID NM_005657,
        as JSON::

            >>> g.get_gene_id("human", "C_RSEQ", "NM_005657")

        Get the GeneProf IDs of all human genes with any ID matching
        "NM_005657" (should, in this case, be same as the previous query)::

            >>> g.get_gene_id("human", "any", "NM_005657")
        """
        format_ = self._check_format(format)

        url = self.url + "/gene.info/gp.id/%s/%s/%s." % (ref, idtype, Id)
        url += format_

        res = self.request(url, format=format_)
        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res


    def get_external_gene_id(self, ref, idtype, Id, format="json"):
        """Translates a GeneProf gene ID into an external identifier or name.

        GeneProf uses sets of gene annotations based on those from Ensembl.
        With this method, you can look up an external name (official gene
        symbol) or one of the supported accession ID types (e.g. Ensembl
        Gene IDs, RefSeq IDs, etc. -- use the :meth:`get_list_idtypes` service
        to find out which types are supported for a dataset) for any given
        internal GeneProf gene ID.

        :param str Id:   The GeneProf ID of a gene (an integer number).
        :param str ref:  The identifier of a public GeneProf reference
            dataset. You may use aliases here. Check the
            :meth:`get_list_reference_datasets`
            service for all  available reference datasets.
        :param str idtype:   The identifier an annotation column storing
            IDs. Check the :meth:`get_list_idtypes` to find out which types
            are supported for a  dataset.
        :param str format:  format requests, one of: json, txt, xml, rdata.

        Get the Ensembl Gene ID(s) of the mouse gene #715, as plain text:

             >>> g.get_external_gene_id("mouse","715", "C_ENSG")

         Get the RefSeq ID(s) of the human gene #2981, as JSON:

             >>> g.get_external_gene_id("mouse","2981", "C_RSEQ")

         Get the name(s) of the human gene #2981, as XML:

             >>> g.get_external_gene_id("mouse","2981", "C_NAME")
        """
        format_ = self._check_format(format)
        url = self.url + "/gene.info/external.id/%s/%s/%s." % (ref, idtype, Id)
        url += format_
        res = self.request(url, format=format_)
        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res

    def get_list_idtypes(self, ref, format="json"):
        """list all the ID types available for a dataset.

        GeneProf reference datasets provide a number of alternative ID annotations (e.g. Ensembl Gene IDs, RefSeq IDs, UniGene IDs, etc.) for each of the genes in the reference annotation. This service simply lists all the ID types available for a dataset.

        :param str ref: identifier of a public GeneProf reference dataset (e.g.
            human)
        :param str format: format, one of: json, txt, xml, rdata.


        ::

            >>> g.get_list_idtypes("mouse")
            >>> g.get_list_idtypes("human")

        """
        format_ = self._check_format(format)
        url = self.url + "/gene.info/list.id.types/%s." % ref
        url += format_
        res = self.request(url, format=format_)
        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res


    def get_expression(self, ref, Id, format="json", with_sample_info=False, type="RPKM"):
        """Alias to :meth:`get_gene_expression`
        
        
        
        
        """
        return self.get_gene_expression(ref, Id, format=format, 
                with_sample_info=with_sample_info, type=type)


    def get_gene_expression(self, ref, Id, format="json",
            with_sample_info=False,  type="RPKM"):
        """Get Gene Expression Values for a Gene

        Retrieves gene expression values for a gene based on public RNA-seq data
        in the GeneProf databases. 
        
        GeneProf's databases contain many
        pre-calculated gene expression values stemming from a reanalyses of a
        large collection of RNA-seq (and similar) experiments. Use this web
        service to retrieve all the expression values for a single gene of
        interest by giving the name of the **reference** dataset the gene
        belongs to and its internal GeneProf gene **Id** -- use the
        :meth:`get_list_reference_datasets`, :meth:`get_gene_id`  and/or
        :meth:`search_genes` methods to look up these identifiers. You may
        retrieve the values either as raw read counts (the
        total number of short reads that were aligned to the gene's locus), RPM
        (reads per million -- the raw counts rescaled to account for differences
        in library size) or RPKM (reads per kilobase million -- like RPM, but
        also accounting for transcript length bias). All gene expression values
        have been calculated using the
        `Calculate Gene Expression module
        <http://www.geneprof.org/GeneProf/help_modules.jsp#module:org.stembio.geneprof.workflow.module.rna.CalculateGeneExpression>`_.
        Full details for the analysis pipeline that was used to calculate each
        value are available from the individual experiments the values come
        from (the JSON and XML output contain a link to the experiment of
        origin).

        :param str Id: GeneProf ID of a gene (an integer number).
        :param str ref: identifier of a public GeneProf reference dataset. You
            may use aliases here. Check the list public references service for
            all available reference datasets.
        :param str format: format requests, one of: json, txt, xml, rdata.
        :param str type: type of values to obtain, one of: RAW | RPM | RPKM
        :param str with_sample_info: Include additional annotations about the
            tissue, cell type, etc. of the expression values.

        Retrieve gene expression values for the mouse gene #715, including additional annotation data:

            >>> g.get_gene_expression("mouse", "715", with_sample_info=True)

        Retrieve raw read count values for the mouse gene #715

            >>> g.get_gene_expression("mouse", "715", type="RAW")

        Retrieve gene expression values for the mouse gene #715 as a tab-delimited text file, including additional annotation data:

            >>> g.get_gene_expression("mouse", "715", format="txt", with_sample_info=True)

        Retrieve gene expression values for the mouse gene #715 as an
        RData file, including additional annotation data:

            >>> g.get_gene_expression("mouse", "715", format="rdata", with_sample_info=True)


        .. plot::
            :include-source:
            :width: 80%

            >>> from bioservices import GeneProf
            >>> import math
            >>> from pylab import hist, title, xlabel, clf, show, ylabel
            >>> g = GeneProf()
            >>> res = g.get_gene_expression("mouse", "715")
            >>> rpkmValues = [x["RPKM"] for x in res]
            >>> logValues = [math.log(x+1,2.) for x in rpkmValues]
            >>> hist(logValues) 
            >>> xlabel('RPKM'); ylabel('Count')
            >>> title( 'Histogram: 715')
            >>> show()

        """
        format_ = self._check_format(format)
        type_ = type
        self.checkParam(type_, ["RAW", "RPM", "RPKM"])

        url = self.url + "/gene.info/expression/%s/%s." % (ref, Id)
        url += format_

        #TODO: params
        params = {}
        params['with-sample-info'] = with_sample_info
        params['type'] = type_
        params = self.urlencode(params)
        url += "?"+ params
        res = self.request(url, format=format_)

        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
            res = res['values']
        return res

    def get_targets_tf(self,ref,Id,format="json", ats="C_NAME",
            include_unbound=False):
        """Retrieve putative target genes for a transcription factor

        Retrieve putative target genes for a transcription factor
        (or other transcriptional regulator) based on public
        ChIP-seq data in the GeneProf databases by querying for the
        targets discovered in all available ChIP-seq experiments
        (identified by the ID of a gene).

        GeneProf's databases contain lots of information about putative
        gene regulatory interactions from a reanalyses of a large
        collection of ChIP-seq experiments. 
        
        Give the name of the reference
        dataset the TF gene belongs to and its internal GeneProf gene ID -- use
        the :meth:`get_list_reference_datasets`, :meth:`get_gene_id` or
        :meth:`search_genes` or :meth:`get_gene_id` methods to look up these 
        identifiers. The assignment of putative
        target genes to TFs has been done by calling enriched binding peaks on
        the aligned ChIP-seq reads using MACS and subsequently assigning the
        peaks to target genes if they were within a permissible window of the
        transcription start site (as by current wizard default: 20kb up- and
        1kb down-stream of the TSS; in an upcoming release of the web service,
        you will be able to redefine these threshold dynamically, so watch this
        space!). The GeneProf workflow modules corresponding to these two steps
        are documented in `Find Peaks with MACS <http://www.geneprof.org/GeneProf/help_modules.jsp#module:org.stembio.geneprof.workflow.module.chipseq.MACS14withFDRFilter>`_ and `Map Regions to Genes  <http://www.geneprof.org/GeneProf/help_modules.jsp#module:org.stembio.geneprof.workflow.module.chipseq.MapRegionsToGenes>`_.
        Full details for the analysis pipeline that was used to calculate each
        value are available from the individual experiments the values come
        from (the JSON and XML output contain a link to the experiment of
        origin). For some TFs there might be more than one dataset available,
        in which case the output returned by the web service will contain the
        status in all availabledatasets (distinguished by the experimental
        sample they belong to, see :meth:`get_list_experiment_samples`.

        :param str Id: The GeneProf ID of a gene/feature (an integer number).
        :param str ref: identifier of a public GeneProf reference dataset.
            Check the list public references service for all available reference
            datasets.
        :param str format: format requests, one of: json, txt, xml, rdata.
        :param str ats: (default C_NAME)  A selection of column IDs (from the
            reference) to be included in the output.
        :param bool include_unbound: include not only putative target genes in
           the output, but also those genes that show now evidence of regulation.


        Get all the putative targets of the mouse TF Smad1 in JSON format::

            >>> g.get_targets_tf("mouse", "9885")

        Get all the putative targets of the human TF MEIS1, also include
        unbound genes for comparison::

            >>> g.get_targets_tf("human", "36958", include_unbound=True)

        Get all the putative targets of the mouse TF Nanog as
        include a column for gene name and Ensembl ID (there are TWO ChIP-seq
        datasets available for this TF!)::

            >>> g.get_targets_tf("mouse", "14899", ats="C_NAME,C_ENSG")

        Get all the putative targets of the human TF MEIS1 as an RData file::

            >>> g.get_targets_tf("human", "36958", format="rdata")


        """
        format_ = self._check_format(format)
        url = self.url + "/gene.info/regulation/binary/by.gene/%s/%s." % (ref, Id)
        url += format_
        params = {}
        params['ats'] = ats
        params['include-unbound'] = include_unbound
        params = self.urlencode(params)
        url += "?"+ params

        res = self.request(url, format=format_)

        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res

    def get_targets_by_experiment_sample(self, ref, Id, format="json", 
        ats="C_NAME", include_unbound=False):
        """Get Targets by Experiment Sample

        Retrieve putative target genes for a transcription factor (or other
        transcriptional regulator) based on public ChIP-seq data in the GeneProf
        databases by querying for the targets discovered in a specific ChIP-seq
        experiment (identified by the ID of a public sample).

        GeneProf's databases contain lots of information about putative gene
        regulatory interactions from a reanalyses of a large collection of
        ChIP-seq experiments. You use this web service to retrieve a list of
        putative target genes for a transcription factor (TF) or other
        DNA-binding protein (incl. histone modifications), by giving the
        identifier of a public GeneProf sample -- use the
        :meth:`get_list_experiment_samples` or
        the :meth:`search_samples` method to look up these identifiers. The
        assignment of putative target genes to TFs has been done by calling
        enriched binding peaks on the aligned ChIP-seq reads using MACS and
        subsequently assigning the peaks to target genes if they were within a
        permissible window of the transcription start site (as by current wizard
        default: 20kb up- and 1kb down-stream of the TSS; in an upcoming release
        of the web service, you will be able to redefine these threshold
        dynamically, so watch this space!). The GeneProf workflow modules
        corresponding to these two steps are documented here: Find Peaks with
        MACS and Map Regions to Genes. Full details for the analysis pipeline
        that was used to calculate each value are available from the individual
        experiments the values come from (the JSON and XML output contain a link
        to the experiment of origin).

        :param str Id: the GeneProf ID of a public sample (an integer number).
        :param str ref: The identifier of a public GeneProf reference dataset.
            You may use aliases here. Check the list public references service
            for all available reference datasets.
        :param str format: format requests, one of: json, txt, xml, rdata.
        :param str ats: (default C_NAME)  A selection of column IDs (from the
            reference) to be included in the output.
        :param bool include_unbound: default False. Include not only putative
            target genes in the output, but also those genes that show now
            evidence of regulation.

        Get all the putative targets of the mouse TF Smad1 in JSON format::

            >>> g.get_targets_by_experiment_sample("mouse", "541")

        Get all the putative targets of the human TF MEIS1 in XML
        format, also include unbound genes for comparison::

            >>> g.get_targets_by_experiment_sample("human", "784", include_unbound=True)

        Get all the putative targets of the mouse TF Smad1 as
        tab-delimited text and include a column for gene name and
        Ensembl ID::

            >>> g.get_targets_by_experiment_sample("mouse", "541", ats="C_NAME,C_ENSG")

        Get all the putative targets of the human TF MEIS1 as an RData file::

            >>> g.get_targets_by_experiment_sample("human", "784", format="rdata")


        """
        format_ = self._check_format(format)
        url = self.url + "/gene.info/regulation/binary/by.sample/%s/%s." % (ref, Id)
        url += format_
        params = {}
        params['ats'] = ats
        params['include-unbound'] = include_unbound
        params = self.urlencode(params)
        url += "?"+ params

        res = self.request(url, format=format_)

        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res

    def get_tfas_by_gene(self, ref, Id, format="json", ats="C_NAME",
            include_unbound=False):
        """Get TFAS of a Transcription Factor

        Retrieve transcription factor association strength (TFAS)
        scores for a transcription factor (or other
        transcriptional regulator) based on public ChIP-seq data in the GeneProf
        databases by querying for the data in all available ChIP-seq experiments
        (identified by the ID of a gene).

        Full description:   GeneProf's databases contain lots of information
        about putative gene regulatory interactions from a reanalyses of a large
        collection of ChIP-seq experiments. You use this web service to retrieve
        a list of TFAS scores for a transcription factor (TF) or other
        DNA-binding protein, by giving the name of the reference dataset the TF
        gene belongs to and its internal GeneProf gene ID -- use the list
        reference datasets, get GeneProf ID and/or search genes services to look
        up these identifiers. 'TFAS' (= transcription factor association
        strength) scores are continuous values that give an indication of how
        strongly a transcription factor (or other DNA-binding protein) is
        associated with a target gene. The TFAS is calculated as a function of
        the intensity and the distance of all binding sites (ChIP-seq peaks)
        near a gene, for details, please refer to the publication by Ouyang et
        al. (PubMed: 19995984). We use as an intensity score the fold-change
        enrichment of the ChIP-seq signal over the control background as
        calculated by MACS in conjunction with calling peaks for the input
        ChIP-seq data. The GeneProf workflow modules corresponding to these two
        steps are documented here: Find Peaks with MACS and Calculate TFAS. Full
        details for the analysis pipeline that was used to calculate each value
        are available from the individual experiments the values come from (the
        JSON and XML output contain a link to the experiment of origin). For
        some TFs there might be more than one dataset available, in which case
        the output returned by the web service will contain the status in all
        available datasets (distinguished by the experimental sample they belong
        to, see list public samples service).

        :param str Id:  The GeneProf ID of a gene/feature (an integer number).
        :param str Ref: The identifier of a public GeneProf reference dataset.
            You may use aliases here. Check the list public references service
            for all available reference datasets.
        :param str format: format requests, one of: json, txt, xml, rdata.
        :param str ats: (default C_NAME) a selection of column IDs (from
            the reference) to be included in the output.

        Get all TFAS scores for the mouse TF Smad1 in JSON format:

            >>> g.get_tfas_by_gene("mouse", "9885")

        Get all TFAS scores for the human TF MEIS1 in XML format, also include
        unbound genes for comparison:

            >>> g.get_tfas_by_gene("human", "36958", format="xml",
                    include_unbound=True)

        Get all TFAS scores the mouse TF Nanog as tab-delimited text and include
        a column for gene name and Ensembl ID (there are TWO ChIP-seq datasets
        available for this TF!):

            >>> g.get_tfas_by_gene("mouse", "14899", format="txt",
                    ats="C_NAME,C_ENSG")

        Get all TFAS scores for the human TF MEIS1 as an RData file:

            >>> g.get_tfas_by_gene("human", "36958", format="rdata")



        """
        format_ = self._check_format(format)
        url = self.url + "/gene.info/regulation/tfas/by.gene/%s/%s." % (ref, Id)
        url += format_
        params = {}
        params['ats'] = ats
        params['include-unbound'] = include_unbound
        params = self.urlencode(params)
        url += "?"+ params

        res = self.request(url, format=format_)

        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res

    def get_tfas_by_sample(self, ref, Id, format="json", ats="C_NAME",
            include_unbound=False):
        """Get TFAS by Experiment Sample

        Retrieve a list of TFAS scores for a transcription factor (TF) or 
        other DNA-binding protein, by giving the identifier of a public GeneProf
        sample -- use the list public samples or the :meth:`search_samples`
        to look up these identifiers. TFAS (transcription factor association
        strength) scores are continuous values that give an indication of how
        strongly a transcription factor (or other DNA-binding protein) is
        associated with a target gene. The TFAS is calculated as a function of
        the intensity and the distance of all binding sites (ChIP-seq peaks)
        near a gene, for details, please refer to the publication by Ouyang et
        al. (PubMed: 19995984). We use as an intensity score the fold-change
        enrichment of the ChIP-seq signal over the control background as
        calculated by MACS in conjunction with calling peaks for the input
        ChIP-seq data. The GeneProf workflow modules corresponding to these two
        steps are documented here: Find Peaks with MACS and Calculate TFAS. Full
        details for the analysis pipeline that was used to calculate each value
        are available from the individual experiments the values come from (the
        JSON and XML output contain a link to the experiment of origin).

        :param str Id: The GeneProf ID of a public sample (an integer number).
        :param str ref: identifier of a public GeneProf reference dataset.
            You may use aliases here. Check the list public references service
            for all available reference datasets.
        :param str format: format requests, one of: json, txt, xml, rdata.
        :param str ats: (default C_NAME) a selection of column IDs (from the
            reference) to be included in the output.

        Get TFAS scores for the mouse TF Smad1 in JSON format:

            >>> g.get_tfas_by_sample("mouse", 541)

        Get TFAS scores for the human TF MEIS1 in XML format, also include
        unbound genes for comparison:

            >>> g.get_tfas_by_sample("human", "784", format="xml",
                    >>> include_unbound=True)

        Get TFAS scores for the mouse TF Smad1 as tab-delimited text and include
        a column for gene name and Ensembl ID:

            >>> g.get_tfas_by_sample("mouse", "541", format="txt",
                ats="C_NAME,C_ENSG")

        Get TFAS scores for the human TF MEIS1 as an RData file:

            >>> g.get_tfas_by_sample("human", "784")

        """
        format_ = self._check_format(format)
        url = self.url + "/gene.info/regulation/tfas/by.sample/%s/%s." % (ref, Id)
        url += format_
        params = {}
        params['ats'] = ats
        params['include-unbound'] = include_unbound
        params = self.urlencode(params)
        url += "?"+ params

        res = self.request(url, format=format_)

        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res


    def get_tf_by_target_gene(self, ref, Id, format="json", ats="C_NAME",
            with_sample_info=False):
        """Get Transcription Factors by Target Gene

        Retrieve transcription factors (and other regulatory inputs) putatively
        targeting a specific gene, based on public ChIP-seq data in the GeneProf
        databases.

        GeneProf's databases contain lots of information about putative gene
        regulatory interactions from a reanalyses of a large collection of
        ChIP-seq experiments. You use this web service to retrieve a list of
        transcription factors and other DNA-binding proteins that might possible
        be regulating a gene of interest, by giving the name of the reference
        dataset the gene belongs to and its internal GeneProf gene ID -- use the
        :meth:`get_list_reference_datasets`, :meth:`get_gene_id` or
        :meth:`search_genes` to look up these identifiers. The assignment 
        of putative target genes to
        TFs has been done by calling enriched binding peaks on the aligned
        ChIP-seq reads using MACS and subsequently assigning the peaks to target
        genes if they were within a permissible window of the transcription
        start site (as by current wizard default: 20kb up- and 1kb
        down-stream of the TSS; in an upcoming release  of the web service, you
        will be able to redefine these threshold dynamically, so watch
        this space!). The GeneProf workflow modules corresponding to these two
        steps are documented here: Find Peaks with MACS and Map Regions to
        Genes. Full details for the analysis pipeline that was used to calculate
        each value are available from the individual experiments the values come
        from (the JSON and XML output contain a link to the experiment of
        origin).

        :param str Id: GeneProf ID of a gene (an integer number).
        :param str ref: The identifier of a public GeneProf reference dataset.
            see :meth:`get_list_reference_datasets` and :attr:`valid_species`
            for list of public references.
        :param bool format: format requests, one of: json, txt, xml, rdata.
        :param bool with_sample_info: default false. Include additional 
            annotations about the  tissue, cell type, etc. of the expression
            values.

        Get information about factors putatively targeting gene #715 in JSON
            format, including additional annotation data::

            >>> g.get_tf_by_target_gene("mouse", "715", with_sample_info=True)

        Get information about factors putatively targeting gene #715 in
        XML format, including additional annotation data::

            >>> g.get_tf_by_target_gene("mouse", "715", format="xml",
                    with_sample_info=True)

        Get information about factors putatively targeting gene #715
        as a tab-delimited text file::

            >>> g.get_tf_by_target_gene("mouse", "715", format="txt")

        Get information about factors putatively targeting gene
        715 as an RData file, including additional annotation
        data::

            >>> g.get_tf_by_target_gene("mouse", "715", format="rdata",
                    with_sample_info=True)

        """
        format_ = self._check_format(format)
        url = self.url + "/gene.info/regulation/binary/by.target/%s/%s." % (ref, Id)
        url += format_
        params = {}
        params['with-sample-info'] = with_sample_info
        params = self.urlencode(params)
        url += "?"+ params

        res = self.request(url, format=format_)

        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res

    def get_tfas_scores_by_target(self, ref, Id, format="json",
            with_sample_info=False):
        """Get TFAS Scores by Target Gene

        Rerieve a list of Transcription Factor Association Strength scores 
        quantitating  the association between transcription factors
        (TFs) and other DNA-binding proteins and a gene of interest, by giving
        the name of the reference dataset the gene belongs to and its internal
        GeneProf gene ID -- use the :meth:`get_list_reference_datasets`, 
        :meth:`get_gene_id`, :meth:`search_genes` to look up these identifiers.

        TFAS scores are continuous values that give an indication of how 
        strongly a transcription factor (or other
        DNA-binding protein) is associated with a target gene. The TFAS is
        calculated as a function of the intensity and the distance of all
        binding sites (ChIP-seq peaks) near a gene, for details, please refer to
        the publication by Ouyang et al. (PubMed: 19995984). We use as an
        intensity score the fold-change enrichment of the ChIP-seq signal over
        the control background as calculated by MACS in conjunction with calling
        peaks for the input ChIP-seq data. The GeneProf workflow modules
        corresponding to these two steps are documented here: 
        `Find Peaks with MACS <http://www.geneprof.org/GeneProf/help_modules.jsp#module:org.stembio.geneprof.workflow.module.chipseq.MACS14withFDRFilter>`_ and `Calculate TFAS <http://www.geneprof.org/GeneProf/help_modules.jsp#module:org.stembio.geneprof.workflow.module.chipseq.TranscriptionFactorAssociationStrength>`_
            
        Full details for the analysis pipeline that was  used to calculate each
        value are available from the individual experiments the values come from
        (the JSON and XML output contain a link to the experiment of origin).

        :param str Id: The GeneProf ID of a gene (an integer number).
        :param str ref:   The identifier of a public GeneProf reference
            dataset. You may use aliases here. Check the list public references
            service for all available reference datasets.
        :param str format: format requests, one of: json, txt, xml, rdata.
        :param bool with-sample-info: (default false) Include additional
            annotations about the tissue, cell type, etc. of the expression values.

        Get TFAS scores to gene #715 in JSON format, including additional
        annotation data::

            >>> g.get_tfas_scores_by_target("mouse", "715", with_sample_info=True)

        Get TFAS scores to gene #715 in XML format, including additional
        annotation data::

            >>> g.get_tfas_scores_by_target("mouse", 715, with_sample_info=True)


        Get TFAS scores to gene #715 as a tab-delimited text file::

            >>> g.get_tfas_scores_by_target("mouse", 715, format="txt")

        Get TFAS scores to gene #715 as an RData file, including additional
        annotation data::

            >>> g.get_tfas_scores_by_target("mouse", "715", format="rdata",
                with_sample_info=True)
        """
        format_ = self._check_format(format)
        url = self.url + "/gene.info/regulation/tfas/by.target/%s/%s." % (ref, Id)
        url += format_
        params = {}
        params['with-sample-info'] = with_sample_info
        params = self.urlencode(params)
        url += "?"+ params

        res = self.request(url, format=format_)

        if format_ == "json":
            # TODO: special characters to be removed.
            res = json.loads(res.replace("\\", ""))
        return res

    def get_metadata_usr(self):
        """Metadata about a User


        """
        raise NotImplementedError

    def get_data(self, Id, format=None, sep="\t", gz=False, ats=None, key=None):
        """Retrieves data as Plain Text Files (TXT)

        Retrieve data from a GeneProf dataset as plain
        text (optionailly compressed as GZIP). Retrieves the entire contents
        of an arbitrary GeneProf dataset as a tab-delimited, plain text file.
        The dataset of interest is identified by its GeneProf accession ID
        (something of the form gpDS_XXX_XXX_XXX_X). You can get a list of
        datasets belonging to a certain experiment of interest using the
        :meth:`get_metadata_experiment` method, or you can use the
        :meth:`search_datasets` method to query datasets globally.

        :param str id: identifier of the dataset of interest. Either the entire
            accession ID (e.g. gpDS_11_385_44_1) or just the dataset-specific
            part (e.g. 11_385_44_1).
        :param str gz: compressed the data (if format is not rdata).
        :param str ats: (default displayed columns) A selection of column IDs
            to be included in the output.
        :param str sep: (default is tabulation \t) symbol to be used as
            a column separator.
        :param str key: optional WebAPI key, required to access non-public data.

        Retrieve data from all visible columns of the dataset gpDS_11_119_18_1
        (example RNA-seq data):

            >>> g.get_data("11_119_18_1", format="txt", gz=True)

        Retrieve only the Ensembl Gene IDs and RPKM values from the same
        dataset:

            >>> g.get_data("11_119_18_1", format="txt", gz=True,
                    ats="C_ENSG,C_11_119_16_1_RPKM0,C_11_119_16_1_RPKM1,C_11_119_16_1_RPKM2,C_11_119_16_1_RPKM3")

        """
        format_ = format
        if format_ not in ["txt", "xls", "xml", "rdata"]:
            raise ValueError("format must be txt,xls,xml,rdata")
        url = self.url + "/data/%s." % (Id)
        url += format_
        if gz:
            if format_ == "rdata":
                raise ValueError("rdata and gz cannot be combined.")
            url += ".gz"
        params = {}
        if ats:
            params['ats'] = ats
        if key:
            params['key'] = key
        if sep!="\t":
            params['sep'] = sep
        if len(params):
            params = self.urlencode(params)
            url += "?" + params
        res = self.request(url, format=format_)
        return res

    def get_chromosome_names(self, Id, format="json", key=None):
        """Get Chromosome Names

        Retrieve the IDs and names of all chromosomes in a genomic dataset. This
        service can only be used for genomic datasets, i.e. for datasets with
        type GENOMIC_REGIONS or REFERENCE.

        The names in genome databases use to refer to chromosomes, even
        of well-known organisms, are not always the same. For example, the
        mitochondrial (pseudo-)chromosome is usally called 'chrMT' in Ensembl,
        but 'chrM' in the UCSC databases. The data as :meth:`get_bed_files`
        or as `WIG <get_wig_files>`_ might therefore require you to rename
        the experiments in the
        output, before using them with other applications. 
        
        :param str Id: The identifier of the dataset of interest. Either the
            entire accession ID (e.g. gpDS_11_385_44_1) or just the
            dataset-specific part (e.g. 11_385_44_1).
        :param str key: optional WebAPI key, required to access non-public data.

        Get all chromosomes for the mouse reference dataset in plain text
        format::

            >>> g.get_chromosome_names("pub_mm_ens58_ncbim37", format="txt")

        Get all chromosomes for the human reference dataset in JSON format::

            >>> g.get_chromosome_names("pub_hs_ens59_grch37")

        Get the chromosome names from the ChIP-seq peaks dataset gpDS_11_3_7_2
        in  XML format::

            >>> g.get_chromosome_names("11_3_7_2", format="xml")

        """
        format_ = self._check_format(format)

        url = self.url + "/data/chromosome.names/%s." % (Id)
        url += format_

        params = {}
        if key:
            params['key'] = key
            params = self.urlencode(params)
            url += "?" + params
        res = self.request(url, format=format_)
        return res


    def get_bed_files(self, Id, chromosome=None, key=None, filter_column=None,
            with_track_description=True, only_distinct=False):
        """Retrieve Genomic Data as compressed BED Files (gzipped)

        Retrieves genomic data as `BED <http://genome.ucsc.edu/FAQ/FAQformat.html#format1>`_  
        files in compressed gzipped format. It works only for datasets with type
        GENOMIC_REGIONS  i.e. those containing genomic data! The dataset of
        interest is identified by its GeneProf accession ID (something of the
        form  gpDS_XXX_XXX_XXX_X). You can get a list of datasets belonging to a
        certain experiment of interest using the metadata for an experiment
        service :meth:`get_metadata_experiment`, or you can use the search 
        datasets service to query datasets globally. The maximum size of 
        datasets retrieved without an
        API key is restricted to 10,000,000 entries. 
        
        .. note:: chromosomes in the output BED can be
            dynamically renamed in order to make the names compatible with other
            applications (that's because, unfortunately, not all genome 
            databases use the same names, see also the 
            :meth:`get_chromosome_names` method).

        :param str Id: The identifier of the dataset of interest. Either
            the entire  accession ID (e.g. gpDS_11_385_44_1) or just the
            dataset-specific part (e.g. 11_385_44_1).
        :param str chromosome:  An optional parameter that may be used to rename
            chromosomes in the output. The value should be comma-separated map
            from  chromosome ID to its name in the output, where key and value
            are to be  separated with a hyphen (-), e.g. 1-chr1,2-chr2,3-chr12.
            Any chromosome  not mentioned in the map will not be exported, so
            you can use this as a  filtering mechanism, too. Use the Get
            Chromosome Names service to get a  list of all the available
            chromosome in a dataset with their default  names.
        :param bool filter_column: The ID of a column / annotation type
            holding  boolean flags. Only entries for which this boolean flag
            is true will be  exported.
        :param bool bool with-track-description: Include a track description
            header.
        :param bool only_distinct: Export only one entry if there are
            multiple with the same coordinates.
        :param str key:  An optional WebAPI key, required to access non-public
            data.

        Retrieve ChIP-seq peaks for FoxA1 from dataset gpDS_11_3_7_2:

            >>> g.get_bed_files("11_3_7_2")

        Retrieve only the ChIP-seq peaks on chromosome 3 for FoxA1 from dataset
        gpDS_11_3_7_2:

            >>> g.get_bed_files("11_3_7_2", chromosome="3-chr3")

        Retrieve gene coordinates from the zebrafish reference dataset without a
        track header:

            >>> g.get_bed_files("zebrafish", with_track_description=False)

        Retrieve only the ChIP-seq peaks for Stat3 (identified by the column
        $C_11_12_125_2_14_TFBS) from a dataset containing peaks for many
        different factors (gpDS_11_12_125_2):

            >>> g.get_bed_files("11_12_125_2",
                    filter_column="C_11_12_125_2_14_TFBS")

        """
        url = self.url + "/data/"
        if chromosome:
            self.url + chromosome + "/"
        url += "%s.bed.gz" % (Id)

        params = {}
        if with_track_description == False:
            params['with-track-description'] = False
        if filter_column != None:
            params['filter-column'] = filter_column
        if only_distinct == True:
            params['only-distinct'] = only_distinct
        if key:
            params['key'] = key
            params = self.urlencode(params)
            url += "?" + params
        res = self.request(url, format="txt")
        return res

    def get_wig_files(self, Id, chromosome=None, key=None, frag_length=-1,
            with_track_description=True, only_distinct=False, bin_size=25):
        """Genomic Data as WIG Files (WIG)

        Retrieves the entire contents of a genomic GeneProf
        dataset in WIG file format. This will only work for dataset of type
        GENOMIC_REGIONS, i.e. those containing genomic data! The dataset of
        interest is identified by its GeneProf accession ID (something of the
        form     gpDS_XXX_XXX_XXX_X). You can get a list of datasets belonging
        to a certain experiment of interest using the
        :meth:`get_metadata_experiment` service, or you can use the
        :meth:`search_datasets` method to query datasets globally. The maximum
        size of datasets retrieved without an API key is restricted to
        10,000,000 entries. With an API key, the maximum size is unlimited.
        
        .. note:: chromosomes in the output BED can be
            dynamically renamed in order to make the names compatible with other
            applications (because not all genome databases
            use the same names, see also the get chromosome names service).

        :param str Id: The identifier of the dataset of interest. Either
            the entire accession ID (e.g. gpDS_11_385_44_1) or just the
            dataset-specific part (e.g. 11_385_44_1).
        :param str chromosome: An optional parameter that may be used to
            rename chromosomes in the output. The value should be
            comma-separated map from chromosome ID to its name in the
            output, where key and value are to be separated with a
            hyphen (-), e.g. 1-chr1,2-chr2,3-chr12. Any chromosome not
            mentioned inthe map will not be exported, so you can use
            this as a filtering mech anism, too. Use the Get Chromosome
            Names service to get a list of all the available chromosome
            in a dataset with their default names.
        :param bool with-track-description: (default True) Include a track
            description header.
        :param bool only_distinct: Include only one entry in the
            coverage count if there are multiple with the same
            coordinates. (default False)
        :param int frag-length: (default -1)  The "fragment length" to
            calculate the  coverage with, use -1 to use the actual size
            of the regions.
        :param int bin-size: (default 25)  The bin size / resolution
            of the tracks.
        :param str key: An optional WebAPI key, required to access
            non-public data.

        Retrieve genomic coverage data from a RNA-seq assay of gene
        expression in human liver gpDS_11_58_16_2:

            >>> g.get_wig_files("11_58_16_2")

        Retrieve genomic coverage data from a ChIP-seq experiment for
        Smad1 (gpDS_11_12_112_2), using only distinct alignments:

            >>> g.get_wig_files("11_12_112_2", with_track_description=False,
                only_distinct=True, frag_length=200)
        """
        url = self.url + "/data/"
        if chromosome:
            self.url + chromosome + "/"
        url += "%s.wig.gz" % (Id)

        params = {}
        if with_track_description == False:
            params['with-track-description'] = False
        if frag_length != -1:
            params['frag-length'] = frag_length
        if bin_size != -1:
            params['bin-size'] = bin_size
        if only_distinct == True:
            params['only-distinct'] = only_distinct
        if key:
            params['key'] = key
            params = self.urlencode(params)
            url += "?" + params
        res = self.request(url, format=format_)
        return res


    def get_fasta(self, Id, key=None):
        """Sequence Data as FASTA Files (FASTA, gzipped)

        Retrieves the entire contents of a nucleotide sequence dataset in
        `FASTA <http://en.wikipedia.org/wiki/FASTA_format>`_
        format. This will only work for dataset of type SEQUENCES, i.e. those
        containing sequence data! The dataset of interest is identified by its
        GeneProf accession ID (something of the form
        gpDS_XXX_XXX_XXX_X). You can get a list of datasets belonging to a
        certain experiment of interest using the :meth:`get_metadata_experiment`
        service, or you can use the :meth:`search_datasets` method to query
        datasets  globally. The maximum size of datasets retrieved
        without an API key is restricted to 10,000,000 entries. With an
        API key, the maximum size is unlimited.

        :param str Id: The identifier of the dataset of interest. Either the
            entire accession ID (e.g. gpDS_11_385_44_1) or just the
            dataset-specific part (e.g.11_385_44_1).
        :param str key: optional WebAPI key, required to access non-public data.

        Retrieve unprocessed Tag-seq sequence data from gpDS_11_385_6_1:

            >>> g.get_fasta("11_385_6_1")

        """
        url = self.url + "/data/%s.fasta.gz" % (Id)
        params = {}
        if key:
            params['key'] = key
            params = self.urlencode(params)
            url += "?" + params
        res = self.request(url)
        return res

    def get_fastq(self, Id, key=None):
        """Sequence Data as FASTQ Files (FASTQ)

        Same as :meth:`get_fasta` but for FASTQ format.

        Retrieve unprocessed Tag-seq sequence data from gpDS_11_385_6_1:

            >>> g.get_fastq("11_385_6_1")

        """
        url = self.url + "/data/%s.fastq.gz" % (Id)
        params = {}
        if key:
            params['key'] = key
            params = self.urlencode(params)
            url += "?" + params
        res = self.request(url)
        return res












def get_env(section, option):
    import os
    import ConfigParser
    cf = ConfigParser.ConfigParser()

    homedir = os.getenv("HOME") 
    bioservices_home = homedir + os.sep + ".bioservices"
    config_file = bioservices_home + os.sep + "bioservices.cfg"

    if os.path.isdir(bioservices_home) == False:
        os.mkdir(bioservices_home)
    if os.path.isfile(config_file) == False:
        cf.add_section("chemspider")
        cf.set("chemspider", "token", "") 
        fh = open(config_file, "w")
        cf.write(fh)
        fh.close()
        raise ValueError("No token found for chemspider. Create one on http//www.chemspider.com and provide it when creating instance of ChemSpider")
    else:
        cf.read(config_file)
        value = cf.get(section, option)
        if value=="":
            raise ValueError("No token found for chemspider. Create one on http//www.chemspider.com and provide it when creating instance of ChemSpider")
        return value







# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: services.py 302 2013-12-16 17:38:55Z cokelaer $
"""


"""
from __future__ import print_function

from SOAPpy import SOAPProxy, WSDL
import urllib
import urllib2
import platform
#import logging
import easydev
from  easydev import checkParam, Logging


__all__ = ["Service", "WSDLService", "RESTService", "BioServicesError"]


class BioServicesError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)


class Service(Logging):
    """Base class for WSDL and REST classes


    .. seealso:: :class:`RESTService`, :class:`WSDLService`

    """

    response_codes = {
        200 : 'OK',
        400 : 'Bad Request',
        404 : 'Not found'}

    def __init__(self, name, url=None, verbose=True):
        """.. rubric:: Constructor


        :param str name: a name for this service
        :param str url: its URL
        :param bool verbose: prints informative messages if True (default is
            True)

        All instances have an attribute called :attr:`~Service.logging` that is an instance 
        of the :mod:`logging` module. It can be used to print information, warning, 
        error messages::

            self.logging.info("informative message")
            self.logging.warning("warning message")
            self.logging.error("error message")
 
        The attribute :attr:`~Service.debugLevel`  can be used to set the behaviour 
        of the logging messages. If the argument verbose is True, the debugLebel 
        is set to INFO. If verbose if False, the debugLevel is set to WARNING. 
        However, you can use the :attr:`debugLevel` attribute to change it to 
        one of DEBUG, INFO, WARNING, ERROR, CRITICAL. debugLevel=WARNING means 
        that only WARNING, ERROR and CRITICAL messages are shown.

        """
        if verbose == True:
            level = "INFO"
        else:
            level = "WARNING"
        super(Service, self).__init__(level=level)

        self._url = url
        #self.url = url
        self.name = name
        self._easyXMLConversion = True

        # used by HGNC where some XML contains non-utf-8 characters !! 
        self._fixing_unicode = False
        self._fixing_encoding = "utf-8"


    def _get_url(self):
        return self._url
    def _set_url(self, url):
        # something more clever here to check the URL e.g. starts with http
        if url!= None:
            url = url.rstrip("/")
            self._url = url
    url = property(_get_url, _set_url, doc="URL of this service")

    def _get_easyXMLConversion(self):
        return self._easyXMLConversion
    def _set_easyXMLConversion(self, value):
        if type(value) != bool:
            raise TypeError("value must be a boolean value (True/False)")
        self._easyXMLConversion = value
    easyXMLConversion = property(_get_easyXMLConversion, 
        _set_easyXMLConversion, doc="""If True, xml output from a request are converted to
easyXML object (Default behaviour).""")

    def easyXML(self, res):
        """Use this method to convert a XML document into an
            :class:`~bioservices.xmltools.easyXML` object

        The easyXML object provides utilities to ease access to the XML
        tag/attributes.

        Here is a simple example starting from the following XML

        .. doctest::

            >>> from bioservices import *
            >>> doc = "<xml> <id>1</id> <id>2</id> </xml>"
            >>> s = Service("name")
            >>> res = s.easyXML(doc)
            >>> res.findAll("id")
            [<id>1</id>, <id>2</id>]

        """
        from bioservices import xmltools
        return xmltools.easyXML(res, encoding=self._fixing_encoding,
                    fixing_unicode=self._fixing_unicode)

    def urlencode(self, params):
        """Returns a string compatible with a URL request.

        :param dict params: a dictionary. Keys are parameters.

        The pair of key/value are converted into a single string by concatenated
        the "&key=value" string for each key/value in the dictionary. 

        ::

            >>> params = {'a':1, 'b':2}
            >>> urlencode(params)
            "a=1&b=2"

        .. note:: returns "a=1&b=2" or "b=2&a=1" since dictionary are not ordered. Note
            that the first parameter is not preceded by a & sign that you will need
            to add. 

        """
        if isinstance(params, dict)==False:
            raise TypeError("Params must be a dictionary.")
        import urllib
        postData = urllib.urlencode(params)
        return postData

    def checkParam(self, param, valid_values):
        """Simple utility to check that a parameter has a valid value 

        :param param:
        :param valid_values:

        Calls :func:`easydev.tools.checkParam`

        ::

            checkParam(aboolean, [True, False])
            checkParam(mode, ["mean", "std", "skew"])
        """        
        easydev.tools.checkParam(param, valid_values)

    def __str__(self):
        txt = "This is an instance of %s service" % self.name
        return txt

    def pubmed(self, Id):
        """Open a pubmed Id into a browser tab

        :param Id: a valid pubmed Id in string or integer format.

        The URL is a concatenation of the pubmed URL
        http://www.ncbi.nlm.nih.gov/pubmed/ and the provided Id.

        """
        url = "http://www.ncbi.nlm.nih.gov/pubmed/"
        import webbrowser
        webbrowser.open(url + str(Id))

    def onWeb(self, url):
        """Open a URL into a browser"""
        import webbrowser
        webbrowser.open(url)


class WSDLService(Service):
    """Class dedicated to the web services based on WSDL/SOAP protocol.

    .. seealso:: :class:`RESTService`, :class:`Service`

    """

    def __init__(self, name, url, verbose=True, lib="soappy"):
        """.. rubric:: Constructor

        :param str name: a name e.g. Kegg, Reactome, ...
        :param str url: the URL of the WSDL service
        :param bool verbose: prints informative messages

        The :attr:`serv` give  access to all WSDL functionalities of the service.

        The :attr:`methods` is an alias to self.serv.methods and returns 
        the list of functionalities.

        """
        assert lib in ["suds", "soappy"],\
            "library used to connect to the WSDL service must either sud or soappy (default)"
        super(WSDLService, self).__init__(name, url, verbose=verbose)

        #self.serv = SOAPProxy(self.url) # what's that ? can we access to a method directly ?
        self.logging.info("Initialising %s service (WSDL)" % self.name)

        try:
            #: attribute to access to the methods provided by this WSDL service
            if lib == "soappy":
                self.serv = WSDL.Proxy(self.url)
            elif lib == "suds":
                serv = WSDL.Proxy(self.url)
                from suds.client import Client
                self.suds = Client(self.url)
                self.serv = self.suds.service
                self.serv.methods = serv.methods.copy()
        except Exception, e:
            self.logging.error("Could not connect to the service %s " % self.url)
            raise Exception

    def _get_methods(self):
        return sorted(self.serv.methods.keys())
    methods = property(_get_methods, doc="returns methods available in the WSDL service")

    def _get_dump_out(self):
        return self.serv.soapproxy.config.dumpSOAPOut
    def _set_dump_out(self, value):
        self.serv.soapproxy.config.dumpSOAPOut = value
    dumpOut = property(_get_dump_out, _set_dump_out, 
        doc="set the dumpSOAPOut mode of the SOAP proxy (0/1)")

    def _get_dump_in(self):
        return self.serv.soapproxy.config.dumpSOAPIn
    def _set_dump_in(self, value):
        self.serv.soapproxy.config.dumpSOAPIn = value
    dumpIn = property(_get_dump_in, _set_dump_in, 
        doc="set the dumpSOAPIn mode of the SOAP proxy (0/1)")



class RESTService(Service):
    """Class to manipulate REST service

    You can request an URL with this class that also inherits from
    :class:`Service`.

    """

    def __init__(self, name, url=None, verbose=True):
        """.. rubric:: Constructor

        :param str name: a name e.g. Kegg, Reactome, ...
        :param str url: the URL of the REST service
        :param str debugLevel: logging level. See :class:`Service`.

        """
        super(RESTService, self).__init__(name, url, verbose=verbose)
        self.last_response = None
        self.logging.info("Initialising %s service (REST)" % self.name)
        try:
            import urllib
            urllib.urlopen(self.url)
        except Exception, e:
            self.logging.critical("The URL (%s) provided cannot be reached" % self.url)
            print(e)

    def getUserAgent(self):
        import os
        self.logging.info('getUserAgent: Begin')
        urllib_agent = 'Python-urllib/%s' % urllib2.__version__
        clientRevision = ''
        clientVersion = ''
        user_agent = 'EBI-Sample-Client/%s (%s; Python %s; %s) %s' % (
            clientVersion, os.path.basename( __file__ ), 
            platform.python_version(), platform.system(),
            urllib_agent
        )
        self.logging.info('getUserAgent: user_agent: ' + user_agent)
        self.logging.info('getUserAgent: End')
        return user_agent



    def request(self, path, format="xml", baseUrl=True):
        """Send a request via an URL to the web service.

        :param str path: the request will be formed as self.url+/+path 
        :param str format: If the expected output is in XML
            format then it will be converted with :meth:`easyXML`. If the 
            returned document is not in XML, format should be set to any other 
            value.
        :param str baseUrl: By default, the path argument is appended to the 
            :attr:`url` attribute (the main REST URL). However, sometimes, you
            would prefer to provide the entire URL yourself (e.g. in psicquic service)
            If so, set this baseUrl argument to False. 


        .. note:: this is a HTTP GET request 
        """
        import socket
        socket.setdefaulttimeout(1000)
        if path.startswith(self.url):
            url = path
        elif baseUrl == False:
            url = path
        else:
            url = self.url + "/" +  path

        self.logging.info("REST.bioservices.%s request begins" % self.name)
        self.logging.info("--Fetching url=%s" % url)

        try:
            res = urllib2.urlopen(url).read()
            if format=="xml":
                if self.easyXMLConversion:
                    #logging.warning("--Conversion to easyXML"),
                    try:
                        res = self.easyXML(res)
                    except Exception,e :
                        self.logging.warning(e)
                        self.logging.warning("--Conversion to easyXML failed. returns the raw response"),
            self.last_response = res
            return res
        except Exception, e:
            self.logging.error(e)
            self.logging.error("An exception occured while reading the URL")
            self.logging.error(url)
            self.logging.error("Error caught within bioservices. Invalid requested URL ? ")
            raise

    def _request_timeout(self, path, format="xml", baseUrl=True, trials=3,
            timeout=5):
        import socket
        socket.setdefaulttimeout(timeout)
        level = self.debugLevel
        self.debugLevel="ERROR"
        res = None
        for i in range(0,trials):
            try:
                res = self.request(path, format=format, baseUrl=baseUrl)
            except socket.timeout:
                print("Time out. Trying again %s/%s" % (i+1, trials))
            except Exception:
                raise Exception
            else:
                break
        if res == None:
            self.logging.error("URL could not be fetched %s" % path)
        return res
            



    def requestPost(self, requestUrl, params, extra=None):
        """request with a POST method.

        :param str requestUrl: the entire URL to request
        :param dict params: the dictionary of parameter/value pairs
        :param str extra: an additional string to add after the params if
            needed. Could be usefule if a parameter/value can not be added to the
            dictionary. For instance is a parameter has several values

        .. todo:: parameter paranName with a list of values [v1,v2] can be interpreted as
            paramName=v1&paramName=v2

        .. note:: this is a HTTP POST request 
        .. note:: use only by ::`ncbiblast` service so far.
        """
        import sys
        requestData = urllib.urlencode(params)
        print(requestData)
        if extra != None:
            requestData += extra
        # Concatenate the two parts.
        # Errors are indicated by HTTP status codes.
        try:
            # Set the HTTP User-agent.
            user_agent = self.getUserAgent()
            http_headers = { 'User-Agent' : user_agent }
            req = urllib2.Request(requestUrl, None, http_headers)
            # Make the submission (HTTP POST).
            print(req)
            reqH = urllib2.urlopen(req, requestData)
            jobId = reqH.read()
            reqH.close()
        except urllib2.HTTPError, ex:
            # Trap exception and output the document to get error message.
            print(sys.stderr, ex.read())
            raise
        return jobId


def get_bioservices_env(section, option):
    import os
    import ConfigParser
    cf = ConfigParser.ConfigParser()

    homedir = os.getenv("HOME") 
    bioservices_home = homedir + os.sep + ".bioservices"
    config_file = bioservices_home + os.sep + "bioservices.cfg"

    if os.path.isdir(bioservices_home) == False:
        os.mkdir(bioservices_home)
    if os.path.isfile(config_file) == False:
        cf.add_section("chemspider")
        cf.set("chemspider", "token", "")
        fh = open(config_file, "w")
        cf.write(fh)
        fh.close()
        raise ValueError("No token found for chemspider. Create one on http//www.chemspider.com and provide it when creating instance of ChemSpider")
    else:
        cf.read(config_file)
        value = cf.get(section, option)
        if value=="":
            raise ValueError("No token found for chemspider. Create one on http//www.chemspider.com and provide it when creating instance of ChemSpider")
        return value






from bioservices.chembldb import *
import unittest


class test_Chembl(ChEMBLdb):

    def __init__(self):
        super(test_Chembl, self).__init__(verbose=False)
        self.default_extension = "xml"
        try:
            self.default_extension = "xmlf"
            assert False
        except:
            assert True

    def test_api_status(self):
        self.api_status()

    def test_get_compounds_by_chemblId(self):
        self.get_compounds_by_chemblId("CHEMBL1")
        self.get_compounds_by_chemblId("CHEMBL1.json")
        self.get_compounds_by_chemblId(self._chemblId_example)

    def test_get_target_uniprotId(self):
        self.get_target_by_uniprotId("Q13936")

    def _test_get_compounds_activities(self):
        self.get_compounds_activities("CHEMBL1")

    def test_get_compounds_by_SMILES(self):
        self.get_compounds_by_SMILES(self._smiles_example + ".json")


    def test_get_compounds_containing_SMILES(self):
        self.get_compounds_containing_SMILES(self._smiles_example + ".json")

    def test_get_compounds_similar_to_SMILES(self):
        res = self.get_compounds_similar_to_SMILES(self._smiles_example + "/70.json")
        assert len(res['compounds'])>=1

    def test_get_target_by_chemblId(self):
        self.get_target_by_chemblId("CHEMBL2477")

    def test_get_compounds_activities(self):
        self.get_compounds_activities("CHEMBL2")
        self.get_compounds_activities("CHEMBL2.json")

    def test_get_target_bioactivities(self):
        self.get_target_bioactivities("CHEMBL240.json")

    def test_get_target_by_chemblId(self):
        resjson = self.get_target_by_chemblId(self._target_chemblId_example)

    def test_get_assay_bioactivities(self):
        res = self.get_assay_bioactivities(self._assay_example)

    def test_get_all_targets(self):
        self.get_all_targets()

    def test_get_assays(self):
        self.get_assay_bioactivities("CHEMBL1217643")
        self.get_assay_by_chemblId("CHEMBL1217643")

    @unittest.skip
    def test_inspect(self): 
        self.inspect("CHEMBL240")

    def test_image(self):
        self.get_image_of_compounds_by_chemblId(
            self._image_chemblId_example,
            self._image_dimension_example, view=False)




from bioservices import eutils




class test_EUtils(object):

    @classmethod
    def setup_class(klass):
        klass.e = eutils.EUtils(verbose=True)

    def test_espell(self):
        ret = self.e.serv.run_eSpell(db="omim", term="aasthma+OR+alergy")
        assert ret.CorrectedQuery == 'asthma or allergy'

    def test_taxonomy(self):
        ret = self.e.taxonomy("9606")
        ret.Taxon.TaxId
        ret.Taxon.ScientificName

    def test_snp(self):
        self.e.snp("123")


    def test_databases(self):
        assert "protein" in self.e.databases


    def test_summary(self):
        ret = self.e.ESummary("taxonomy", "9606,9913")


    def test_espell(self):
        ret = self.e.ESpell(db="omim", term="aasthma+OR+alergy")
        print ret.Query
        assert ret.Query == 'aasthma OR alergy'
        assert ret.CorrectedQuery == 'asthma or allergy'

    def test_elink(self):
        ret = self.e.ELink("pubmed", "pubmed", Ids="20210808", cmd="neighbor_score")

    def test_einfo(self):
        ret = self.e.EInfo("taxonomy")
        ret.Count

    def test_gquery(self):
        ret = self.e.EGquery("asthma")
        [(x.DbName, x.Count) for x in ret.eGQueryResult.ResultItem if x.Count!='0']

    def test_efetch(self):
        ret = self.e.EFetch("omim", "269840")


        ret1 = self.e.EFetch("sequences", "34577063", retmode="text", rettype="fasta", stand=1)
        ret2 = self.e.EFetch("sequences", "34577063", retmode="text", rettype="fasta", stand=2)
        self.e.EFetch("sequences", "34577063", retmode="text", rettype="fasta",strand=2,seq_start=10, seq_stop=20)



#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: biomodels.py 207 2013-06-12 15:48:05Z cokelaer $
"""This module provides a class :class:`~BioDBNet` to access to BioDBNet WS.


.. topic:: What is BioDBNet ?

    :URL: http://www.ebi.ac.uk/biomodels-main/
    :Service: http://www.ebi.ac.uk/biomodels-main/services/BioDBNetWebServices?wsdl
    :Citations: http://www.ncbi.nlm.nih.gov/pubmed/20587024    

    .. highlights::

        "BioDBNet Database is a repository hosting computational models of biological
        systems. A large number of the provided models are published in the
        peer-reviewed literature and manually curated. This resource allows biologists
        to store, search and retrieve mathematical models. In addition, those models can
        be used to generate sub-models, can be simulated online, and can be converted
        between different representational formats. "

        -- From BioDBNet website, Dec. 2012


Some keywords used in this module:


"""
from bioservices.services import WSDLService
import SOAPpy

__all__ = ["BioDBNet"]




class BioDBNet(WSDLService):
    """Interface to the `BioDBNet <http://www.ebi.ac.uk/biomodels>`_ service 

    ::

        >>> from bioservices import *
        >>> s = BioDBNet()
        >>> model = s.getModelSBMLById('BIOMD0000000299')

    The number of models available can be retrieved easily as well as the model IDs::

        >>> len(s)
        >>> s.modelsId

    Most of the BioDBNet WSDL are available. There are functions added to
    the original interface such as :meth:`extra_getReactomeIds`.


    """
    _url = 'http://biodbnet.abcc.ncifcrf.gov/webServices/bioDBnet.wsdl'
    def __init__(self, verbose=True):
        """.. rubric:: Constructor

        :param bool verbose:

        """
        super(BioDBNet, self).__init__(name="BioDBNet", url=BioDBNet._url, verbose=verbose)

    def _interpret_input_db(self, inputValues):
        if isinstance(inputValues, list):
            inputValues = ",".join(inputValues)
            return inputValues
        elif isinstance(inputValues, str):
            return inputValues
        else:
            raise NotImplementedError

    def _interpret_output_db(self, input_db, output_db):
        if isinstance(output_db, list):
            outputs = ",".join(output_db)
        else:
            outputs = output_db
        inputResult = self.getInputs()
        #getOutputsForInput method
        outputResult = self.getOutputsForInput(input_db)
        for output in outputs.split(","):
            if output not in outputResult:
                print(output + " not found")
                print(outputResult)
                raise Exception
        return outputs



    def db2db(self, input_db, output_db, inputValues, taxon=9606):
        """Retrieves the models which are associated to the provided Taxonomy text.

        :param str text: free (Taxonomy based) text
        :return:  list of model identifiers 

        input_db = 'Ensembl Gene ID' 
        output_db = ['Gene Symbol, Ensembl Protein ID']
        inputValues = 'ENSG00000121410, ENSG00000171428' 

        """
        inputValues = self._interpret_input_db(inputValues)
        outputs = self._interpret_output_db(input_db, output_db)

        #dbPath = 'Ensembl Gene ID->Gene ID->Homolog - Mouse Gene ID->Ensembl Gene ID' 
        #getDirectOutputsForInput method
        #directOutputResult = self.getDirectOutputsForInput(input_db)

        #db2db method
        if taxon:
            taxonId = str(taxon)
            params = SOAPpy.structType({'input': input_db, 'inputValues':
                inputValues, 'outputs': outputs, 'taxonId': taxonId}) 
        else:

            params = SOAPpy.structType({'input': input, 'inputValues': 
                inputValues, 'outputs': outputs, 'taxonId': ''})

        res = self.serv.db2db(params)


    def dbFind(self, input_db, inputValues, taxon="9606"):
        inputValues = self._interpret_input_db(inputValues)
        taxonId = str(taxon)
        params = SOAPpy.structType({'input': input_db, 'inputValues':
            inputValues, 'taxonId': taxonId}) 
        return self.serv.dbFind(params)
        

    def dbOrtho(self, input_db, output_db, inputValues, input_taxon, output_taxon):
        raise NotImplementedError
        inputValues = self._interpret_input_db(inputValues)
        outputs = self._interpret_output_db(input_db, output_db)

        taxon1 = str(input_taxon)
        taxon2 = str(output_taxon) # could be a list ?

        params = SOAPpy.structType({'input': input, 'output': output,
            'inputValues': inputValues,  'inputTaxon': taxon1, 'outputTaxon':
            taxon2})
        
        return self.serv.dbOrtho(params)

    def dbReport(self, input_db, inputValues, taxon=9606):
        inputValues = self._interpret_input_db(inputValues)
        taxonId = str(taxon)
        params = SOAPpy.structType({'input': input_db, 'inputValues':
            inputValues, 'taxonId': taxonId}) 
        return self.serv.dbReport(params)

    def dbWalk(self , dbPath, inputValues, taxon=9606):
        dbPath = 'Ensembl Gene ID->Gene ID->Homolog - Mouse Gene ID->Ensembl Gene ID'
        inputValues = self._interpret_input_db(inputValues)
        taxonId = str(taxon)
        params = SOAPpy.structType({'dbPath': dbPath, 'inputValues':
            inputValues, 'taxonId': taxonId})
        return self.serv.dbWalk(params)

    def getDirectOutputsForInput(self, input_db):
        return self.serv.getDirectOutputsForInput(input_db).split(",")

    def getInputs(self):
        return self.serv.getInputs().split(",")

    def getOutputsForInput(self, input_db):
        if input_db not in self.getInputs():
            raise ValueError("Invalid input database provided")
        return self.serv.getOutputsForInput(input_db).split(",")






from bioservices import *


def test_biogrid():
    b = BioGRID(query=["map2k4","akt1"],taxId = "9606")
    b.biogrid.interactors
    b = BioGRID(query=["mtor","akt1"],taxId="9606",exP="two hybrid")
    b = BioGRID(query="mtor",taxId="9606")
    b.biogrid.interactors





import time
from bioservices import Kegg, UniProt, KeggParser, UniChem, BioDBNet, HGNC
from easydev import Logging
try:
    import pandas as pd
except:
    pass

import  os


class Mapper(Logging):
    """Accepted code:

        uniprot


    m = Mapper()
    # HGNC
    df_hgnc = m.get_all_hgnc_into_df()
    df_hgnc.to_pickle("mapper_hgnc.dat")

    # KEGG
    df_kegg1 = m.get_all_kegg_into_df1()
    df_kegg2 = m.get_all_kegg_into_df2()

    uniq_keggid = 

    """
    kegg_dblinks  = ["IMGT", "Ensembl", "HGNC", "HPRD", "NCBI-GI", "OMIM", "NCBI-GeneID", "UniProt", "Vega"]
    hgnc_dblink =  ['EC','Ensembl', 'EntrezGene', 'GDB', 'GENATLAS',
            'GeneCards', 'GeneTests', 'GoPubmed', 'H-InvDB', 'HCDM', 'HCOP',
            'HGNC', 'HORDE', 'IMGT_GENE_DB', 'INTERFIL', 'IUPHAR', 'KZNF',
            'MEROPS', 'Nucleotide', 'OMIM', 'PubMed', 'RefSeq', 'Rfam',
            'Treefam', 'UniProt', 'Vega', 'miRNA', 'snoRNABase']


    def __init__(self, verbosity="INFO"):
        super(Mapper, self).__init__(level=verbosity)
        self.logging.info("Initialising the services")
        self.logging.info("... uniprots")
        self._uniprot_service = UniProt()

        self.logging.info("... KEGG")
        self._kegg_service = KeggParser(verbose=False)

        self.logging.info("... HGNC")
        self._hgnc_service = HGNC()

        self.logging.info("... UniChem")
        self._unichem_service = UniChem()

        self.logging.info("...BioDBNet")
        self._biodbnet = BioDBNet()

    def _uniprot2refseq(self, name):
        """

        There are 2 refseq alias: REFSEQ_NT_ID and P_REFSEQ_AC.

        Here, we use the first one to agree with wikipedia
        http://en.wikipedia.org/wiki/Protein_Kinase_B

        """
        return self._uniprot_service.mapping(fr="ACC", to="REFSEQ_NT_ID", query="P31749")

    def _update_uniprot_xref(self, df, 
            xref=["HGNC_ID", "ENSEMBLE_ID",  "P_ENTREZGENEID"]):
        """Update the dataframe using Uniprot to map indices onto cross
        reference databases


        """
        for ref in xref:
            print("Processing %s " % ref)
            res = self._uniprot_service.multi_mapping("ACC", ref,
                    list(df.index), timeout=10, ntrials=5)
            if "%s__uniprot_mapping" % ref not in df.columns:
                thisdf = pd.DataFrame({"%s__uniprot_mapping": res.values()},
                        index=res.keys())
                df = df.join(thisdf)
            else:
                for index in df.index:
                    if index in res.keys():
                        df.ix[index]["%s__uniprot_mapping" % ref] = res[index]

    def get_data_from_biodbnet(self, df_hgnc):
        """keys are unique Gene names
        
        input is made of the df based on HGNC data web services

        uniprot accession are duplicated sometimes. If som this is actually the
        iprimary accession entry and all secondary ones.


        e.g. ,
        
        ABHD11 >>>> Q8N723;Q8NFV2;Q8NFV3;Q6PJU0;Q8NFV4;H7BYM8;Q8N722;Q9HBS8 ABHDB_HUMAN Alpha/beta hydrolase domain-containing protein 11
        correspond actually to the primary one : Q8NFV4

        """
        b = biodbnet.BioDBNet()
        res2 = b.db2db("Gene Symbol", ["HGNC ID", "UniProt Accession", "UniProt Entry Name", "UniProt Protein Name", "KEGG Gene ID", "Ensembl Gene ID"], 
                res.keys()[0:2000])

        import pandas as pd
        import StringIO
        c = pd.read_csv(StringIO.StringIO(res2), delimiter="\t", index_col="Gene Symbol")
        return c


class MapperBase(object):
    def __init__(self):
        pass

    def build_dataframe(self):
        raise NotImplementedError

    def to_csv(self):
        raise NotImplementedError

    def read_csv(self):
        raise NotImplementedError


class HGNCMapper(object):
    hgnc_dblink =  ['EC','Ensembl', 'EntrezGene', 'GDB', 'GENATLAS',
            'GeneCards', 'GeneTests', 'GoPubmed', 'H-InvDB', 'HCDM', 'HCOP',
            'HGNC', 'HORDE', 'IMGT_GENE_DB', 'INTERFIL', 'IUPHAR', 'KZNF',
            'MEROPS', 'Nucleotide', 'OMIM', 'PubMed', 'RefSeq', 'Rfam',
            'Treefam', 'UniProt', 'Vega', 'miRNA', 'snoRNABase']
    def __init__(self):
        self._hgnc_service = HGNC()
        self.alldata = self.load_all_hgnc(self)
        self.df = self.build_dataframe()

    def load_all_hgnc(self):
        """keys are unique Gene names"""
        print("Fetching the data from HGNC first. May take a few minutes"),
        self.alldata = self._hgnc_service.mapping_all()
        print("done")

    def build_dataframe(self):
        # simplify to get a dictionary of dictionary
        data = {k1:{k2:v2['xkey'] for k2,v2 in self.alldata[k1].iteritems()} for k1 in self.alldata.keys()}
        dfdata = pd.DataFrame(data)
        dfdata = dfdata.transpose()
        # rename to tag with "HGNC"
        dfdata.columns = [this + "__HGNC_mapping" for this in dfdata.columns]
        print("a dataframe was built using HGNC data set and saved in attributes  self._df_hgnc")
        return dfdata



class KEGGMapper(object):
    """

    """
    kegg_dblinks  = ["IMGT", "Ensembl", "HGNC", "HPRD", "NCBI-GI", "OMIM", "NCBI-GeneID", "UniProt", "Vega"]
    def __init__(self, verbose=True):
        self._kegg_service = KeggParser(verbose=verbose)

        print("Reading all data")
        self.alldata = self.load_all_kegg_entries()
        self.entries = sorted(self.alldata.keys())
        self.df = self.build_dataframe()

    def build_dataframe(self):

        names = ['class', 'definition', 'disease', 'drug_target',
                'module', 'motif', 'name', 'orthology', 'pathway', 'position', 'structure']

        N = len(self.entries)
        # build an empty dataframe with relevant names
        data = {}
        # for the dblinks
        for this in self.kegg_dblinks:
            data.update({"%s_kegg" % this: [None] * N})

        # and other interesting entries
        for name in names:
            #e.g. name == position
            data[name] = [self.alldata[entry][name] if name in self.alldata[entry].keys() else None for entry in self.entries]

        df = pd.DataFrame(data, index=self.entries)

        # scan again to fill the df with dblinks
        for index, entry in enumerate(self.entries):
            res = self.alldata[entry]['dblinks']
            for key in res.keys():
                if key in self.kegg_dblinks:
                    # fill df_i,j
                    df.ix[entry][key+"_kegg"] = res[key]
                else:
                    raise NotImplementedError("Found an unknown key in KEGG dblink:%s" % key)

        return df

    def load_all_kegg_entries(self, filename="kegg_gene.dat"):
        if os.path.isfile(filename):
            import pickle
            results = pickle.load(open(filename, "r"))
            return results
        # TODO:
        # donwload from a URL  
        print("could not find kegg data. fetching data from website if possible")
        print("Fetchning list of genes for hsa")
        names = self._kegg_service.list("hsa")
        names = names.strip().split("\n")
        names = [x.split("\t")[0] for x in names]
        # Fetches the KEGG results using multicore to send several requests at the same time

        mcresults = test_func(names)
        self.mcresults = mcresults
        # here are the entries to be used as keys
        entries = ["hsa:"+x['entry'].split()[0] for x in mcresults]

        results = {}
        for entry, result in zip(entries, mcresults):
            results[entry] = result

        import pickle
        pickle.dump(results, open("kegg_gene.dat","w"))
        return results

from bioservices import KeggParser
kegg = KeggParser(verbose=False)

def test_func(names):
    from easydev import MultiProcessing
    t = MultiProcessing(verbose=False, maxcpu=8)
    for name in names:
        t.add_job(keggfunc, name)
    t.run()
    return t.results

def keggfunc(name):
    global kegg
    try:
        id_ = kegg.get(name)
        res = kegg.parse(id_)
        return res
    except:
        return None









from bioservices import UniProt
from easydev import Logging










class Mapping(object):
    """


        Could use unichem, uniprot, kegg, chembldb
        Could be HGNC


    """


    def __init__(self):
        self._uniprot = UniProt()
        self._mapping['uniprot'] = self._uniprot._mapping
        self.databases = {}

    def map(self):
        pass

    def search(self):
        pass



from bioservices import pathwaycommons



def test_search_queries():
    pc2 = pathwaycommons.PathwayCommons(verbose=False)
    pc2.search("Q06609")
    pc2.search("brca2", type="proteinreference", organism="homo sapiens",
            datasource="pid")
    pc2.search("name:'col5a1'", type="proteinreference", organism=9606)
    pc2.search("a*", page=3)


    #http://www.pathwaycommons.org/pc2/get?uri=http://identifiers.org/uniprot/Q06609
def test_get():
    pc2 = pathwaycommons.PathwayCommons(verbose=False)
    pc2.get("col5a1")
    pc2.get("http://identifiers.org/uniprot/Q06609")


def test_idmapping():
    pc2 = pathwaycommons.PathwayCommons(verbose=False)
    pc2.idmapping("BRCA2")
    pc2.idmapping(["TP53", "BRCA2"])

def test_top_pathways():
    pc2 = pathwaycommons.PathwayCommons(verbose=False)
    res = pc2.top_pathways()


def test_graph(): 
    pc2 = pathwaycommons.PathwayCommons(verbose=False)
    res = pc2.graph(source="http://identifiers.org/uniprot/P20908",
            kind="neighborhood", format="EXTENDED_BINARY_SIF")

    #res = pc2.graph(source="P20908", kind="neighborhood")


    #res = pc2.graph(source="COL5A1", kind="neighborhood")
    #res = pc2.graph(kind="neighborhood", source="COL5A1")


def test_traverse():
     pc2 = pathwaycommons.PathwayCommons(verbose=False)
     res = pc2.traverse(
        uri=['http://identifiers.org/uniprot/P38398', 'http://identifiers.org/uniprot/Q06609'], 
        path="ProteinReference/organism")
     #res = pc2.traverse(
     #       uri="http://identifiers.org/uniprot/Q06609",
     #       path="ProteinReference/entityReferenceOf:Protein/name")
     #res = pc2.traverse(
     #        "http://identifiers.org/uniprot/P38398",
     #        path="ProteinReference/entityReferenceOf:Protein")
     #res =pc2.traverse(uri=["http://identifiers.org/uniprot/P38398",
     #       "http://identifiers.org/taxonomy/9606"],
     #       path="Named/name")


test_traverse()




import urllib2
import threading



class RequestURL(threading.Thread):
    """
    Thread checking URLs.
    No need for communication
    """

    def __init__(self, url):
        """
        Constructor.

        @param urls list of urls to check
        @param output file to write urls output
        """
        threading.Thread.__init__(self)
        self.url = url
        self.results = None

    def run(self):
        """
        Thread run method. Check URLs one by one.
        """
        req = urllib2.Request(self.url)
        try:
            d = urllib2.urlopen(req)
        except urllib2.URLError, e:
            print 'URL %s failed: %s' % (url, e.reason)
        self.results = d.read()
        #print 'write done by %s' % self.name
        #print 'URL %s fetched by %s' % (self.url, self.name)


_urls = [
   "http://rest.kegg.jp/get/path:hsa00010",
   "http://rest.kegg.jp/get/path:hsa00020",
   "http://rest.kegg.jp/get/path:hsa00030",
   "http://rest.kegg.jp/get/path:hsa00040",
   "http://rest.kegg.jp/get/path:hsa00051"
]

class RequestMultiURL(object):
    def __init__(self):
        self.threads = []
    def reset(self):
        self.threads = []
    def add_url(self, url):
        t = RequestURL(url)
        self.threads.append(t)
    def start(self):
        for t in self.threads:
            t.start()
    def is_alive(self):
        s = sum([t.is_alive() for t in self.threads])
        if s > 0:
            return True
        else:
            return False

    def wait(self):
        import time
        while self.is_alive():
            time.sleep(1)
    def get_results(self):
        return [t.results for t in self.threads]


#s = RequestMultiURL()
#for u in _urls:
#    s.add_url(u)
#s.start()
#s.wait()
#s.get_results()



from bioservices import BioDBNet

inputValues = 'ENSG00000121410, ENSG00000171428'

def test_db2db():
    b = BioDBNet()
    b.db2db("UniProt Accession", "Gene ID", "P43403")

    
def test_dbfind():
    b = BioDBNet()
    b.dbFind("Ensembl Gene ID", ['ENSG00000121410', 'ENSG00000171428'])
   

def test_dbortho():
    b = BioDBNet()
    pass


def test_dbReport():
    #b.dbReport("UniProt Accession", "P43403")
    pass

def test_dbWalk():
    b = BioDBNet()
    b.dbWalk('Ensembl Gene ID->Gene ID->Homolog - Mouse Gene ID->Ensembl Gene ID', "ENSG00000121410", 9606)







from bioservices.quickgo import *



class test_quickGO(QuickGO):

    def __init__(self):
        super(test_quickGO, self).__init__(verbose=False)

    def test_annotation_wrong_format(self):
        try:
            res = self.Annotation(tax='9606', format='tsddddddv')
            assert False
        except:
            assert True

    def test_annotation_format_col_compatibility(self):
        # if col provided, format can be only tsv
        try:
            res = self.Annotation(tax='9606', format='fasta', col="evidence")
            assert False
        except:
            assert True

    def test_annotation_wrong_limit(self):
        try:
            res = self.Annotation(tax='9606', format='tsv', limit=-1)
            assert False
        except:
            assert True

        try:
            res = self.Annotation(tax='9606', format='tsv', limit="dummy")
            assert False
        except TypeError:
            assert True
        except:
            assert False

    def test_annotation_no_protein_and_goid(self):
        try:
            self.Annotation(format='tsv', col="ref,evidence",ref='PMID:*')
            assert False
        except ValueError:
            assert True

    def test_annotation_evidence(self):
        self.Annotation(protein='P12345', format='tsv', col="ref,evidence", evidence="IDA")
        self.Annotation(protein='P12345', format='tsv', col="ref,evidence", evidence=["IDA"])
        try:
            self.Annotation(protein='P12345', format='tsv',
                col="ref,evidence",evidence=1)
            assert False
        except:assert True

    def test_annotation_aspect(self):
        self.Annotation(protein='P12345', format='tsv', col="ref,evidence",aspect='F')
        self.Annotation(protein='P12345', format='tsv', col="ref,evidence",aspect='C')
        self.Annotation(protein='P12345', format='tsv', col="ref,evidence",aspect='P')
        try:
            self.Annotation(protein='P12345', format='tsv', col="ref,evidence",aspect='dummy')
            assert False
        except:
            assert True


    def test_annotation_source(self):
        self.Annotation(protein='P12345', format='tsv',
            col="ref,evidence",ref='PMID:*', source="UniProtKB")
        self.Annotation(protein='P12345', format='tsv',
            col="ref,evidence",ref='PMID:*', source=["UniProtKB"])
        try:
            self.Annotation(protein='P12345', format='tsv',
                col="ref,evidence",ref='PMID:*', source=111)
            assert False    
        except:
            assert True

    def test_annotation_protein(self):
        print self.Annotation(protein='P12345', format='tsv', col="ref,evidence",ref='PMID:*')

    def test_annotation_goid(self):
        print self.Annotation(goid='GO:0003824', format='tsv', col="ref,evidence")

    def test_annotation_ref_PMID(self):
        res = self.Annotation(tax='9606', format='tsv', col="ref",ref="PMID:*")

    def test_annotation_qualifier(self):
        res = self.Annotation(tax='9606', format='tsv', 
            col="ref,evidence,proteinID,goID,proteinTaxon,qualifier",ref="PMID:*", 
            qualifier="NOT")
        res = self.Annotation(tax='9606', format='tsv', 
            col="ref,evidence,proteinID,goID,proteinTaxon,qualifier",ref="PMID:*", 
            qualifier=["NOT"])
        try:
            res = self.Annotation(tax='9606', format='tsv', 
                col="ref,evidence,proteinID,goID,proteinTaxon,qualifier",ref="PMID:*", 
                qualifier=1)
            assert False
        except:assert True

    def test_annotation_qualifier2(self):
        res = set([ x for x in self.Annotation(tax='9606', format='tsv', col="qualifier",ref="PMID:*").split()])
        assert 'NOT' in res

    def test_annotation_termUse(self):
        try:
            res = self.Annotation(tax='9606', format='tsv',
                col="qualifier",ref="PMID:*", termUse="slimdummy")
            assert False
        except:
            assert True

    def test_Term(self):
        self.Term("GO:0003824", format="obo")
        self.Term("GO:0003824", format="mini")
        self.Term("GO:0003824")
        try:
            self.Term("GO:0003824", format="dummy")
            assert False
        except:
            assert True

        try:
            self.Term("G:0003824")
            assert False
        except:
            assert True



# -*- python -*-
#
#  This file is part of BioServices software
#
#  Copyright (c) 2013 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://pypi.python.org/pypi/bioservices
#
##############################################################################
import sys

def __main__():
    ids = sys.argv[1]
    filename = sys.argv[2]
    # TODO: check the validity and format ? 
    try:
        from  bioservices import UniProt
        u = UniProt(verbose=False)
        u.debugLevel = "ERROR"
    except ImportError:
        print("Could not import bioservoces ? Check that it is installed. Try 'pip install bioservices'")

    try:
        fasta = u.searchUniProtId(ids, "fasta")
    except:
        print("An error occured while fetching the FASTA file from uniprot")

    try:
        fh = open(filename, "w")
        fh.write(fasta)
        fh.close()
    except:
        print("could not save the FASTA file")


if __name__ == '__main__':
    __main__()
